from typing import Optional

from ailment.expression import BinaryOp, UnaryOp, Expression
from ailment.statement import Assignment
from ailment import Block
from angr.project import Project
from angr.knowledge_base import KnowledgeBase


class PeepholeOptimizationStmtBase:
    """
    The base class for all peephole optimizations that are applied on AIL statements.
    """

    __slots__ = (
        "project",
        "kb",
        "func_addr",
    )
    project: Optional[Project]
    kb: Optional[KnowledgeBase]
    func_addr: Optional[int]

    NAME = "Peephole Optimization - Statement"
    DESCRIPTION = "Peephole Optimization - Statement"
    stmt_classes = None

    def __init__(self, project: Optional[Project], kb: Optional[KnowledgeBase], func_addr: Optional[int] = None):
        self.project = project
        self.kb = kb
        self.func_addr = func_addr

    def optimize(self, stmt, stmt_idx: int = None, block=None, **kwargs):
        raise NotImplementedError("_optimize() is not implemented.")


class PeepholeOptimizationExprBase:
    """
    The base class for all peephole optimizations that are applied on AIL expressions.
    """

    __slots__ = (
        "project",
        "kb",
        "func_addr",
    )
    project: Optional[Project]
    kb: Optional[KnowledgeBase]
    func_addr: Optional[int]

    NAME = "Peephole Optimization - Expression"
    DESCRIPTION = "Peephole Optimization - Expression"
    expr_classes = None

    def __init__(self, project: Optional[Project], kb: Optional[KnowledgeBase], func_addr: Optional[int] = None):
        self.project = project
        self.kb = kb
        self.func_addr = func_addr

    def optimize(self, expr, **kwargs):
        raise NotImplementedError("_optimize() is not implemented.")

    #
    # Util methods
    #

    @staticmethod
    def find_definition(ail_expr: Expression, stmt_idx: int, block: Block) -> None:
        idx = stmt_idx - 1
        if idx >= 0:
            stmt = block.statements[idx]
            if isinstance(stmt, Assignment):
                if stmt.dst.likes(ail_expr):
                    return stmt.src
        return None

    @staticmethod
    def is_bool_expr(ail_expr):
        if isinstance(ail_expr, BinaryOp):
            if ail_expr.op in {
                "CmpEQ",
                "CmpNE",
                "CmpLT",
                "CmpLE",
                "CmpGT",
                "CmpGE",
                "CmpLTs",
                "CmpLEs",
                "CmpGTs",
                "CmpGEs",
            }:
                return True
        if isinstance(ail_expr, UnaryOp) and ail_expr.op == "Not":
            return True
        return False



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
import unittest

import angr


class TestHookSet(unittest.TestCase):
    def test_hookset(self):
        class Foo:
            def run(self):
                return self.blah()

            def blah(self):  # pylint:disable=no-self-use
                return ["foo"]

            def install_hooks(self, tech):
                angr.misc.HookSet.install_hooks(self, blah=tech.blah)

            def remove_hooks(self, tech):
                angr.misc.HookSet.remove_hooks(self, blah=tech.blah)

        class Bar:
            def blah(self, foo):  # pylint:disable=no-self-use
                return ["bar"] + foo.blah()

        class Baz:
            def blah(self, foo):  # pylint:disable=no-self-use
                return ["baz"] + foo.blah()

        class Coward:
            def blah(self, foo):  # pylint:disable=no-self-use,unused-argument
                return ["coward"]

        foo = Foo()
        assert foo.run() == ["foo"]

        bar = Bar()
        baz = Baz()
        foo.install_hooks(bar)
        foo.install_hooks(baz)
        assert foo.run() == ["baz", "bar", "foo"]

        foo.remove_hooks(bar)
        foo.remove_hooks(baz)
        assert foo.run() == ["foo"]

        coward = Coward()
        foo.install_hooks(coward)
        assert foo.run() == ["coward"]

        foo.remove_hooks(coward)
        assert foo.run() == ["foo"]


if __name__ == "__main__":
    unittest.main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("ndfapi.dll")
prototypes = \
    {
        #
        'NdfCreateIncident': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"pwszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "type": SimTypeInt(signed=False, label="ATTRIBUTE_TYPE"), "Anonymous": SimUnion({"Boolean": SimTypeInt(signed=True, label="Int32"), "Char": SimTypeChar(label="Byte"), "Byte": SimTypeChar(label="Byte"), "Short": SimTypeShort(signed=True, label="Int16"), "Word": SimTypeShort(signed=False, label="UInt16"), "Int": SimTypeInt(signed=True, label="Int32"), "DWord": SimTypeInt(signed=False, label="UInt32"), "Int64": SimTypeLongLong(signed=True, label="Int64"), "UInt64": SimTypeLongLong(signed=False, label="UInt64"), "PWStr": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Guid": SimTypeBottom(label="Guid"), "LifeTime": SimStruct({"startTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "endTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="LIFE_TIME", pack=False, align=None), "Address": SimStruct({"family": SimTypeShort(signed=False, label="UInt16"), "data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 126)}, name="DIAG_SOCKADDR", pack=False, align=None), "OctetString": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="OCTET_STRING", pack=False, align=None)}, name="<anon>", label="None")}, name="HELPER_ATTRIBUTE", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["helperClassName", "celt", "attributes", "handle"]),
        #
        'NdfCreateWinSockIncident': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "SubAuthorityCount": SimTypeChar(label="Byte"), "IdentifierAuthority": SimStruct({"Value": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 6)}, name="SID_IDENTIFIER_AUTHORITY", pack=False, align=None), "SubAuthority": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="SID", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["sock", "host", "port", "appId", "userId", "handle"]),
        #
        'NdfCreateWebIncident': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["url", "handle"]),
        #
        'NdfCreateWebIncidentEx': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["url", "useWinHTTP", "moduleName", "handle"]),
        #
        'NdfCreateSharingIncident': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["UNCPath", "handle"]),
        #
        'NdfCreateDNSIncident': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hostname", "queryType", "handle"]),
        #
        'NdfCreateConnectivityIncident': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["handle"]),
        #
        'NdfCreateNetConnectionIncident': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeBottom(label="Guid")], SimTypeInt(signed=True, label="Int32"), arg_names=["handle", "id"]),
        #
        'NdfCreatePnrpIncident': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["cloudname", "peername", "diagnosePublish", "appId", "handle"]),
        #
        'NdfCreateGroupingIncident': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"iAddressCount": SimTypeInt(signed=True, label="Int32"), "Address": SimTypePointer(SimStruct({"lpSockaddr": SimTypePointer(SimStruct({"sa_family": SimTypeShort(signed=False, label="UInt16"), "sa_data": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 14)}, name="SOCKADDR", pack=False, align=None), offset=0), "iSockaddrLength": SimTypeInt(signed=True, label="Int32")}, name="SOCKET_ADDRESS", pack=False, align=None), offset=0)}, name="SOCKET_ADDRESS_LIST", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CloudName", "GroupName", "Identity", "Invitation", "Addresses", "appId", "handle"]),
        #
        'NdfExecuteDiagnosis': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["handle", "hwnd"]),
        #
        'NdfCloseIncident': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["handle"]),
        #
        'NdfDiagnoseIncident': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypePointer(SimStruct({"pwszDescription": SimTypePointer(SimTypeChar(label="Char"), offset=0), "rootCauseID": SimTypeBottom(label="Guid"), "rootCauseFlags": SimTypeInt(signed=False, label="UInt32"), "networkInterfaceID": SimTypeBottom(label="Guid"), "pRepairs": SimTypePointer(SimStruct({"repair": SimStruct({"guid": SimTypeBottom(label="Guid"), "pwszClassName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszDescription": SimTypePointer(SimTypeChar(label="Char"), offset=0), "sidType": SimTypeInt(signed=False, label="UInt32"), "cost": SimTypeInt(signed=True, label="Int32"), "flags": SimTypeInt(signed=False, label="UInt32"), "scope": SimTypeInt(signed=False, label="REPAIR_SCOPE"), "risk": SimTypeInt(signed=False, label="REPAIR_RISK"), "UiInfo": SimStruct({"type": SimTypeInt(signed=False, label="UI_INFO_TYPE"), "Anonymous": SimUnion({"pwzNull": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ShellInfo": SimStruct({"pwszOperation": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszFile": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszParameters": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszDirectory": SimTypePointer(SimTypeChar(label="Char"), offset=0), "nShowCmd": SimTypeInt(signed=False, label="UInt32")}, name="ShellCommandInfo", pack=False, align=None), "pwzHelpUrl": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDui": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")}, name="UiInfo", pack=False, align=None), "rootCauseIndex": SimTypeInt(signed=True, label="Int32")}, name="RepairInfo", pack=False, align=None), "repairRank": SimTypeShort(signed=False, label="UInt16")}, name="RepairInfoEx", pack=False, align=None), offset=0), "repairCount": SimTypeShort(signed=False, label="UInt16")}, name="RootCauseInfo", pack=False, align=None), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Handle", "RootCauseCount", "RootCauses", "dwWait", "dwFlags"]),
        #
        'NdfRepairIncident': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"repair": SimStruct({"guid": SimTypeBottom(label="Guid"), "pwszClassName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszDescription": SimTypePointer(SimTypeChar(label="Char"), offset=0), "sidType": SimTypeInt(signed=False, label="UInt32"), "cost": SimTypeInt(signed=True, label="Int32"), "flags": SimTypeInt(signed=False, label="UInt32"), "scope": SimTypeInt(signed=False, label="REPAIR_SCOPE"), "risk": SimTypeInt(signed=False, label="REPAIR_RISK"), "UiInfo": SimStruct({"type": SimTypeInt(signed=False, label="UI_INFO_TYPE"), "Anonymous": SimUnion({"pwzNull": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ShellInfo": SimStruct({"pwszOperation": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszFile": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszParameters": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwszDirectory": SimTypePointer(SimTypeChar(label="Char"), offset=0), "nShowCmd": SimTypeInt(signed=False, label="UInt32")}, name="ShellCommandInfo", pack=False, align=None), "pwzHelpUrl": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pwzDui": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="<anon>", label="None")}, name="UiInfo", pack=False, align=None), "rootCauseIndex": SimTypeInt(signed=True, label="Int32")}, name="RepairInfo", pack=False, align=None), "repairRank": SimTypeShort(signed=False, label="UInt16")}, name="RepairInfoEx", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Handle", "RepairEx", "dwWait"]),
        #
        'NdfCancelIncident': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Handle"]),
        #
        'NdfGetTraceFile': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Handle", "TraceFileLocation"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.sim.exec_insn"  # pylint:disable=redefined-builtin

import os
import unittest

import claripy

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestRcr(unittest.TestCase):
    def test_rcr(self):
        p = angr.Project(os.path.join(test_location, "i386", "rcr_test"), auto_load_libs=False)
        result = p.factory.successors(p.factory.entry_state()).successors[0]
        assert claripy.is_true(result.regs.cl == 8)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import os
import logging
import unittest

import angr
from angr.sim_type import SimTypePointer, SimTypeChar

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestDirector(unittest.TestCase):
    def test_execute_address_brancher(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "brancher"), load_options={"auto_load_libs": False})

        pg = p.factory.simulation_manager()

        # initialize the exploration technique
        dm = angr.exploration_techniques.Director(num_fallback_states=1)
        goal = angr.exploration_techniques.ExecuteAddressGoal(0x400594)
        dm.add_goal(goal)
        pg.use_technique(dm)

        pg.explore(find=(0x4005B4,))

        assert len(pg.deprioritized) > 0

    def test_call_function_brancher(self):
        class NonLocal:
            the_state = None
            the_goal = None

        def goal_reached_callback(goal, p, pg):  # pylint:disable=unused-argument
            NonLocal.the_state = p
            NonLocal.the_goal = goal

        p = angr.Project(os.path.join(test_location, "x86_64", "brancher"), load_options={"auto_load_libs": False})

        pg = p.factory.simulation_manager()

        # initialize the exploration technique
        dm = angr.exploration_techniques.Director(
            cfg_keep_states=True, goal_satisfied_callback=goal_reached_callback, num_fallback_states=1
        )
        _ = p.analyses.CFG()
        puts_func = p.kb.functions.function(name="puts")
        goal = angr.exploration_techniques.CallFunctionGoal(puts_func, [(SimTypePointer(SimTypeChar()), ">=20")])
        dm.add_goal(goal)
        pg.use_technique(dm)

        pg.explore(find=(0x40059E,))

        assert len(pg.deprioritized) > 0
        assert len(pg.found) > 0
        assert NonLocal.the_state is not None
        assert NonLocal.the_goal is goal


if __name__ == "__main__":
    logging.getLogger("angr.exploration_techniques.director").setLevel(logging.DEBUG)
    unittest.main()



#!/usr/bin/env python3
from unittest import main, TestCase

import archinfo

from angr.storage.memory_mixins.paged_memory.pages.multi_values import MultiValues
from angr.knowledge_plugins.key_definitions.atoms import Register, SpOffset
from angr.knowledge_plugins.key_definitions.live_definitions import LiveDefinitions


class TestLiveDefinitions(TestCase):
    def setUp(self):
        self.arch = archinfo.arch_arm.ArchARM()

        sp_offset = self.arch.registers["sp"][0]
        self.sp_register = Register(sp_offset, self.arch.bytes)

    def test_get_sp_retrieves_the_value_of_sp_register(self):
        live_definitions = LiveDefinitions(self.arch)

        offset = SpOffset(self.arch.bits, 0)
        address = live_definitions.stack_address(offset.offset)
        sp_value = MultiValues(offset_to_values={0: {address}})

        live_definitions.kill_and_add_definition(self.sp_register, None, sp_value)

        retrieved_sp_value = live_definitions.get_sp()

        self.assertEqual(retrieved_sp_value, live_definitions.stack_offset_to_stack_addr(offset.offset))

    def test_get_sp_fails_if_there_are_different_definitions_for_sp_with_different_values(self):
        # To get multiple definitions of SP cohabiting, we need to create a `LiveDefinitions` via `.merge`:
        # Let's create the "base" `LiveDefinitions`, holding *DIFFERENT* values.
        live_definitions = LiveDefinitions(self.arch)

        offset = SpOffset(self.arch.bits, 0)
        address = live_definitions.stack_address(offset.offset)
        sp_value = MultiValues(offset_to_values={0: {address}})
        other_offset = SpOffset(self.arch.bits, 20)
        other_address = live_definitions.stack_address(other_offset.offset)
        other_sp_value = MultiValues(offset_to_values={0: {other_address}})

        live_definitions.kill_and_add_definition(self.sp_register, 0x0, sp_value)
        other_live_definitions = LiveDefinitions(self.arch)
        other_live_definitions.kill_and_add_definition(self.sp_register, 0x1, other_sp_value)
        # Then merge them.
        live_definitions_with_multiple_sps, _ = live_definitions.merge(other_live_definitions)

        self.assertRaises(AssertionError, live_definitions_with_multiple_sps.get_sp)

    def test_get_sp_retrieves_the_value_of_sp_register_even_if_it_has_several_definitions(self):
        # To get multiple definitions of SP cohabiting, we need to create a `LiveDefinitions` via `.merge`:
        # Let's create the "base" `LiveDefinitions`, holding *THE SAME* values.
        live_definitions = LiveDefinitions(self.arch)

        offset = SpOffset(self.arch.bits, 0)
        address = live_definitions.stack_address(offset.offset)
        sp_value = MultiValues(offset_to_values={0: {address}})

        live_definitions.kill_and_add_definition(self.sp_register, 0x1, sp_value)
        other_live_definitions = LiveDefinitions(self.arch)
        other_live_definitions.kill_and_add_definition(self.sp_register, 0x2, sp_value)
        # Then merge them.
        live_definitions_with_multiple_sps, _ = live_definitions.merge(other_live_definitions)

        retrieved_sp_value = live_definitions_with_multiple_sps.get_sp()

        self.assertEqual(retrieved_sp_value, live_definitions.stack_offset_to_stack_addr(offset.offset))


if __name__ == "__main__":
    main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("wecapi.dll")
prototypes = \
    {
        #
        'EcOpenSubscriptionEnum': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["Flags"]),
        #
        'EcEnumNextSubscription': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SubscriptionEnum", "SubscriptionNameBufferSize", "SubscriptionNameBuffer", "SubscriptionNameBufferUsed"]),
        #
        'EcOpenSubscription': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["SubscriptionName", "AccessMask", "Flags"]),
        #
        'EcSetSubscriptionProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="EC_SUBSCRIPTION_PROPERTY_ID"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "DateTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "PropertyHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"), "Count": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EC_VARIANT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Subscription", "PropertyId", "Flags", "PropertyValue"]),
        #
        'EcGetSubscriptionProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="EC_SUBSCRIPTION_PROPERTY_ID"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "DateTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "PropertyHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"), "Count": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EC_VARIANT", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Subscription", "PropertyId", "Flags", "PropertyValueBufferSize", "PropertyValueBuffer", "PropertyValueBufferUsed"]),
        #
        'EcSaveSubscription': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["Subscription", "Flags"]),
        #
        'EcDeleteSubscription': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["SubscriptionName", "Flags"]),
        #
        'EcGetObjectArraySize': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ObjectArray", "ObjectArraySize"]),
        #
        'EcSetObjectArrayProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="EC_SUBSCRIPTION_PROPERTY_ID"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "DateTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "PropertyHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"), "Count": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EC_VARIANT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ObjectArray", "PropertyId", "ArrayIndex", "Flags", "PropertyValue"]),
        #
        'EcGetObjectArrayProperty': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="EC_SUBSCRIPTION_PROPERTY_ID"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "DateTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "PropertyHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"), "Count": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EC_VARIANT", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ObjectArray", "PropertyId", "ArrayIndex", "Flags", "PropertyValueBufferSize", "PropertyValueBuffer", "PropertyValueBufferUsed"]),
        #
        'EcInsertObjectArrayElement': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ObjectArray", "ArrayIndex"]),
        #
        'EcRemoveObjectArrayElement': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ObjectArray", "ArrayIndex"]),
        #
        'EcGetSubscriptionRunTimeStatus': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="EC_SUBSCRIPTION_RUNTIME_STATUS_INFO_ID"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"BooleanVal": SimTypeInt(signed=True, label="Int32"), "UInt32Val": SimTypeInt(signed=False, label="UInt32"), "DateTimeVal": SimTypeLongLong(signed=False, label="UInt64"), "StringVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "BinaryVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "BooleanArr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "Int32Arr": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "StringArr": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "PropertyHandleVal": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="<anon>", label="None"), "Count": SimTypeInt(signed=False, label="UInt32"), "Type": SimTypeInt(signed=False, label="UInt32")}, name="EC_VARIANT", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["SubscriptionName", "StatusInfoId", "EventSourceName", "Flags", "StatusValueBufferSize", "StatusValueBuffer", "StatusValueBufferUsed"]),
        #
        'EcRetrySubscription': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["SubscriptionName", "EventSourceName", "Flags"]),
        #
        'EcClose': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Object"]),
    }

lib.set_prototypes(prototypes)



import logging
import claripy

from angr.storage.memory_mixins import MemoryMixin

l = logging.getLogger(__name__)


class DataNormalizationMixin(MemoryMixin):
    """
    Normalizes the data field for a store and the fallback field for a load to be BVs.
    """

    def store(self, addr, data, size=None, **kwargs):
        data_bv = self._convert_to_ast(data, size, self.state.arch.byte_width)

        # zero extend if size is greater than len(data_e)
        # TODO move this to the register resolver
        # bit_width = size*self.state.arch.byte_width if isinstance(size, int) else self.state.arch.bits
        # if size is not None and self.category == 'reg' and len(data_bv) < bit_width:
        #    data_bv = data_bv.zero_extend(bit_width - len(data_bv))

        if len(data_bv) % self.state.arch.byte_width != 0:
            raise SimMemoryError("Attempting to store non-byte data to memory")

        super().store(addr, data_bv, size=size, **kwargs)

    def load(self, addr, size=None, fallback=None, **kwargs):
        fallback_bv = self._convert_to_ast(fallback, size, self.state.arch.byte_width) if fallback is not None else None
        return super().load(addr, size=size, fallback=fallback_bv, **kwargs)

    def _convert_to_ast(self, thing, size, byte_width):
        """
        :param thing:       The thing to convert to an AST
        :param size:        The size of the thing in bytes
        :param byte_width:  The size of a byte in bits
        """
        if type(thing) is claripy.ast.BV:
            return thing

        if type(size) is int:
            bits = size * byte_width
        elif getattr(size, "op", None) == "BVV":
            bits = size.args[0] * byte_width
        else:
            bits = None

        if isinstance(thing, str):
            l.warning("Encoding unicode string for memory as utf-8. Did you mean to use a bytestring?")
            thing = thing.encode("utf-8")
        if type(thing) in (bytes, bytearray, memoryview):
            return claripy.BVV(thing)
        elif type(thing) is int:
            if bits is None:
                l.warning("Unknown size for memory data %#x. Default to arch.bits.", thing)
                bits = self.state.arch.bits
            return claripy.BVV(thing, bits)
        elif type(thing) is float:
            if bits == 32:
                return claripy.FPV(thing, claripy.FSORT_FLOAT).raw_to_bv()
            elif bits == 64:
                return claripy.FPV(thing, claripy.FSORT_DOUBLE).raw_to_bv()
            else:
                raise TypeError("Passed float size which is not a float or a double", size)
        else:
            try:
                raw_to_bv = thing.raw_to_bv
            except AttributeError:
                raise TypeError("Bad value passed to memory", thing) from None
            else:
                return raw_to_bv()


from ...errors import SimMemoryError



import cle
import io
import logging
import os
import re
import struct

from .plugin import SimStatePlugin
from ..errors import SimConcreteRegisterError
from archinfo import ArchX86, ArchAMD64

l = logging.getLogger("state_plugin.concrete")
# l.setLevel(logging.DEBUG)


class Concrete(SimStatePlugin):
    def __init__(
        self,
        segment_registers_initialized=False,
        segment_registers_callback_initialized=False,
        whitelist=None,
        fs_register_bp=None,
        already_sync_objects_addresses=None,
    ):
        super().__init__()

        self.segment_registers_initialized = segment_registers_initialized
        self.segment_registers_callback_initialized = segment_registers_callback_initialized

        if not whitelist:
            self.whitelist = []
        else:
            self.whitelist = whitelist

        self.synchronize_cle = False
        self.stubs_on_sync = False

        self.fs_register_bp = fs_register_bp

        if not already_sync_objects_addresses:
            self.already_sync_objects_addresses = []
        else:
            self.already_sync_objects_addresses = already_sync_objects_addresses

    def copy(self, _memo):
        conc = Concrete(
            segment_registers_initialized=self.segment_registers_initialized,
            segment_registers_callback_initialized=self.segment_registers_callback_initialized,
            whitelist=list(self.whitelist),
            fs_register_bp=self.fs_register_bp,
            already_sync_objects_addresses=list(self.already_sync_objects_addresses),
        )
        return conc

    def merge(self, _others, _merge_conditions, _common_ancestor=None):
        pass

    def widen(self, _others):
        pass

    def set_state(self, state):
        SimStatePlugin.set_state(self, state)

    def sync(self):
        """
        Handle the switch between the concrete execution and angr.
        This method takes care of:
        1- Synchronize registers.
        2- Set a concrete target to the memory backer so the memory reads are redirected in the concrete process memory.
        3- If possible restore the SimProcedures with the real addresses inside the concrete process.
        4- Set an inspect point to sync the segments register as soon as they are read during the symbolic execution.
        5- Flush all the pages loaded until now.

        :return:
        """

        def _sync_segments(state):
            """
            Segment registers synchronization is on demand as soon as the
            symbolic execution access a segment register.
            """
            concr_target = state.project.concrete_target

            if isinstance(state.arch, ArchAMD64):
                state.project.simos.initialize_segment_register_x64(state, concr_target)
            elif isinstance(state.arch, ArchX86):
                gdt = state.project.simos.initialize_gdt_x86(state, concr_target)
                state.concrete.whitelist.append((gdt.addr, gdt.addr + gdt.limit))

            state.inspect.remove_breakpoint("reg_read", bp=state.concrete.fs_register_bp)
            state.concrete.segment_registers_initialized = True

            state.concrete.fs_register_bp = None

        l.debug("Sync the state with the concrete memory inside the Concrete plugin")

        # Configure plugin with state options
        if options.SYMBION_SYNC_CLE in self.state.options:
            self.synchronize_cle = True
        if options.SYMBION_KEEP_STUBS_ON_SYNC in self.state.options:
            self.stubs_on_sync = True

        target = self.state.project.concrete_target

        # Sync angr registers with the one getting from the concrete target
        # registers that we don't want to concretize.
        l.debug("Synchronizing general purpose registers")

        to_sync_register = list(filter(lambda x: x.concrete, self.state.arch.register_list))

        for register in to_sync_register:
            # before let's sync all the subregisters of the current register.
            # sometimes this can be helpful ( i.e. ymmm0 e xmm0 )
            if register.subregisters:
                subregisters_names = map(lambda x: x[0], register.subregisters)
                self._sync_registers(subregisters_names, target)

            # finally let's synchronize the whole register
            self._sync_registers([register.name], target)

        if self.synchronize_cle:
            self._sync_cle(target)

        # Synchronize the imported functions addresses (.got, IAT) in the
        # concrete process with ones used in the SimProcedures dictionary
        if self.state.project.use_sim_procedures and not self.state.project.loader.main_object.pic:
            self._sync_simproc()
        else:
            l.debug("SimProc not restored, you are going to simulate also the code of external libraries!")

        # flush the angr memory in order to synchronize them with the content of the
        # concrete process memory when a read/write to the page is performed
        self.state.memory.flush_pages(self.whitelist)
        l.info(
            "Exiting SimEngineConcrete: simulated address %x concrete address %x ",
            self.state.addr,
            target.read_register("pc"),
        )

        # now we have to register a SimInspect in order to synchronize the segments register
        # on demand when the symbolic execution accesses it
        if self.state.project.arch.name in ["X86", "AMD64"] and not self.segment_registers_callback_initialized:
            segment_register_name = self.state.project.simos.get_segment_register_name()
            if segment_register_name:
                self.fs_register_bp = self.state.inspect.b(
                    "reg_read", reg_read_offset=segment_register_name, action=_sync_segments
                )

                self.segment_registers_callback_initialized = True

                l.debug("Set SimInspect breakpoint to the new state!")
            else:
                l.error("Can't set breakpoint to synchronize segments registers, horrible things will happen.")

    def _sync_registers(self, register_names, target):
        for register_name in register_names:
            try:
                reg_value = target.read_register(register_name)
                setattr(self.state.regs, register_name, reg_value)
                l.debug(
                    "Register: %s value: %x ",
                    register_name,
                    self.state.solver.eval(getattr(self.state.regs, register_name), cast_to=int),
                )
            except SimConcreteRegisterError as exc:
                l.debug(
                    "Can't set register %s reason: %s, if this register is not used this message can be ignored",
                    register_name,
                    exc,
                )

    def _sync_cle(self, target):
        def _check_mapping_name(cle_mapping_name, concrete_mapping_name):
            if cle_mapping_name == concrete_mapping_name:
                return True
            else:
                # removing version and extension information from the library name
                cle_mapping_name = re.findall(r"[\w']+", cle_mapping_name)
                concrete_mapping_name = re.findall(r"[\w']+", concrete_mapping_name)
                return (cle_mapping_name[0] == concrete_mapping_name[0]) if len(concrete_mapping_name) else False

        l.debug("Synchronizing CLE backend with the concrete process memory mapping")
        try:
            vmmap = target.get_mappings()
        except NotImplementedError:
            l.critical("Can't synchronize CLE backend using the ConcreteTarget provided.")
            self.synchronize_cle = False  # so, deactivate this feature
            l.debug("CLE synchronization has been deactivated")
            return

        for mapped_object in self.state.project.loader.all_elf_objects:
            binary_name = os.path.basename(mapped_object.binary)

            # this object has already been sync, skip it.
            if binary_name in self.already_sync_objects_addresses:
                continue

            for mmap in vmmap:
                if _check_mapping_name(binary_name, mmap.name):
                    l.debug("Match! %s -> %s", mmap.name, binary_name)

                    # let's make sure that we have the header at this address to confirm that it is the
                    # base address.
                    # That's not a perfect solution, but should work most of the time.
                    result = target.read_memory(mmap.start_address, 0x10)

                    if self.state.project.loader.main_object.check_magic_compatibility(io.BytesIO(result)):
                        if mapped_object.mapped_base == mmap.start_address:
                            # We already have the correct address for this memory mapping
                            l.debug(
                                "Object %s is already rebased correctly at 0x%x", binary_name, mapped_object.mapped_base
                            )
                            self.already_sync_objects_addresses.append(mmap.name)

                            break  # object has been synchronized, move to the next one!

                        # rebase the object if the CLE address doesn't match the real one,
                        # this can happen with PIE binaries and libraries.
                        l.debug(
                            "Remapping object %s mapped at address 0x%x at address 0x%x",
                            binary_name,
                            mapped_object.mapped_base,
                            mmap.start_address,
                        )

                        old_mapped_base = mapped_object.mapped_base
                        mapped_object.mapped_base = mmap.start_address  # Rebase now!

                        # TODO re-write this horrible thing
                        mapped_object.sections._rebase(abs(mmap.start_address - old_mapped_base))  # fix sections
                        mapped_object.segments._rebase(abs(mmap.start_address - old_mapped_base))  # fix segments

                        self.already_sync_objects_addresses.append(mmap.name)
                        break  # object has been synchronized, move to the next one!

    def _sync_simproc(self):
        l.debug("Restoring SimProc using concrete memory")

        for reloc in self.state.project.loader.main_object.relocs:
            if reloc.symbol:  # consider only reloc with a symbol
                l.debug("Trying to re-hook SimProc %s", reloc.symbol.name)
                # l.debug("reloc.rebased_addr: %#x " % reloc.rebased_addr)

                if self.state.project.simos.name == "Win32":
                    func_address = self.state.project.concrete_target.read_memory(
                        reloc.rebased_addr, self.state.arch.bytes
                    )
                    func_address = struct.unpack(self.state.project.arch.struct_fmt(), func_address)[0]
                elif self.state.project.simos.name == "Linux":
                    try:
                        func_address = self.state.project.loader.main_object.plt[reloc.symbol.name]
                    except KeyError:
                        continue
                else:
                    l.info("Can't synchronize simproc, binary format not supported.")
                    return

                l.debug("Function address hook is now: %#x ", func_address)
                self.state.project.rehook_symbol(func_address, reloc.symbol.name, self.stubs_on_sync)

                if self.synchronize_cle and not self.state.project.loader.main_object.contains_addr(func_address):
                    old_func_symbol = self.state.project.loader.find_symbol(reloc.symbol.name)

                    if old_func_symbol:  # if we actually have a symbol
                        owner_obj = old_func_symbol.owner

                        # calculating the new real address
                        new_relative_address = func_address - owner_obj.mapped_base

                        new_func_symbol = cle.backends.Symbol(
                            owner_obj,
                            old_func_symbol.name,
                            new_relative_address,
                            old_func_symbol.size,
                            old_func_symbol.type,
                        )

                        for new_reloc in self.state.project.loader.find_relevant_relocations(old_func_symbol.name):
                            if (
                                new_reloc.symbol.name == new_func_symbol.name
                                and new_reloc.value != new_func_symbol.rebased_addr
                            ):
                                l.debug(
                                    "Updating CLE symbols metadata, moving %s from 0x%x to 0x%x",
                                    new_reloc.symbol.name,
                                    new_reloc.value,
                                    new_func_symbol.rebased_addr,
                                )

                                new_reloc.resolve(new_func_symbol)
                                new_reloc.relocate([])


from ..sim_state import SimState
from .. import sim_options as options

SimState.register_default("concrete", Concrete)



#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use
import math
from unittest import TestCase, main

import claripy
import angr


class TestSqrt(TestCase):
    def test_sqrt_symbolic(self):
        ins_bytes = b"\xf3\x0f\x51\xc9"  # sqrtss  xmm1, xmm1
        proj = angr.load_shellcode(ins_bytes, "amd64", load_address=0)
        state = proj.factory.blank_state(addr=0)

        xmm1 = claripy.FPS("v", claripy.FSORT_FLOAT)
        state.regs.xmm1 = xmm1.to_bv()
        simgr = proj.factory.simgr(state)

        simgr.step(num_inst=1)
        assert len(simgr.active) == 1

        final = simgr.active[0]
        result = final.regs.xmm1[31:0].raw_to_fp()

        a = int(final.solver.eval(xmm1, extra_constraints=(result == 2.0,)))
        assert a == 4
        b = int(final.solver.eval(xmm1, extra_constraints=(result == 4.0,)))
        assert b == 16

    def test_sqrt_concrete(self):
        ins_bytes = b"\xf3\x0f\x51\xc9"  # sqrtss  xmm1, xmm1
        proj = angr.load_shellcode(ins_bytes, "amd64", load_address=0)
        state = proj.factory.blank_state(addr=0)

        xmm1 = claripy.FPV(200000, claripy.FSORT_FLOAT)
        state.regs.xmm1 = xmm1.to_bv()
        simgr = proj.factory.simgr(state)

        simgr.step(num_inst=1)
        assert len(simgr.active) == 1

        final = simgr.active[0]
        result = final.regs.xmm1[31:0].raw_to_fp()

        a = final.solver.eval(result)
        assert abs(a - math.sqrt(200000)) < 0.001

    def test_sqrt_concrete_eager_evaluation(self):
        a = claripy.FPV(2.0, claripy.FSORT_DOUBLE)
        b = claripy.fpSqrt(a)
        assert abs(b.concrete_value - 1.414) < 0.001


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location

test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestCfgClflush(unittest.TestCase):
    def test_cfgfast_clflush(self):
        bin_path = os.path.join(test_location, "x86_64", "igt_stats")
        p = angr.Project(bin_path, auto_load_libs=False)
        # build a CFG of function 0x12190
        cfg = p.analyses.CFG(
            function_starts=(0x412190,),
            # Do not scan the entire binary
            force_complete_scan=False,
            symbols=False,
            function_prologues=False,
        )
        node = cfg.get_any_node(0x4121AA)
        assert node is not None
        assert len(node.successors) == 1

    def test_cfgemulated_clflush(self):
        bin_path = os.path.join(test_location, "x86_64", "igt_stats")
        p = angr.Project(bin_path, auto_load_libs=False)
        # build a CFG of function 0x12190
        cfg = p.analyses.CFGEmulated(starts=(0x412190,))
        node = cfg.get_any_node(0x4121AA)
        assert node is not None
        assert len(node.successors) == 1


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.factory"  # pylint:disable=redefined-builtin

import os
import unittest

import claripy

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestArgv(unittest.TestCase):
    def test_mips(self):
        proj = angr.Project(os.path.join(test_location, "mips", "argv_test"), auto_load_libs=False)
        r_addr = 0x400768

        s = proj.factory.entry_state(args=["aaa", "Yan is a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 1

        s = proj.factory.entry_state(args=["aaa", "Yan is not a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 0

        # symbolic command line argument
        arg = claripy.BVS("arg_2", 50 * 8)
        s = proj.factory.entry_state(args=["aaa", arg], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        found = xpl.found[0]
        conc = found.solver.eval(found.memory.load(found.registers.load("sp"), 400), cast_to=bytes)

        assert b"Yan is a noob" in conc

    def test_mipsel(self):
        proj = angr.Project(os.path.join(test_location, "mipsel", "argv_test"), auto_load_libs=False)
        r_addr = 0x400768
        s = proj.factory.entry_state(args=["aaa", "Yan is a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 1

        s = proj.factory.entry_state(args=["aaa", "Yan is not a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 0

        # symbolic args
        s = proj.factory.entry_state(args=["aaa", claripy.BVS("arg_2", 50 * 8)], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        found = xpl.found[0]
        conc = found.solver.eval(found.memory.load(found.registers.load("sp"), 400), cast_to=bytes)

        assert b"Yan is a noob" in conc

    def test_i386(self):
        proj = angr.Project(os.path.join(test_location, "i386", "argv_test"), auto_load_libs=False)
        r_addr = 0x804845B
        s = proj.factory.entry_state(args=["aaa", "Yan is a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 1

        s = proj.factory.entry_state(args=["aaa", "Yan is not a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 0

        # symbolic args
        s = proj.factory.entry_state(args=["aaa", claripy.BVS("arg_2", 50 * 8)], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        found = xpl.found[0]
        conc = found.solver.eval(found.memory.load(found.registers.load("sp"), 400), cast_to=bytes)

        assert b"Yan is a noob" in conc

    def test_amd64(self):
        proj = angr.Project(os.path.join(test_location, "x86_64", "argv_test"), auto_load_libs=False)
        r_addr = 0x400571
        s = proj.factory.entry_state(args=["aaa", "Yan is a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 1

        s = proj.factory.entry_state(args=["aaa", "Yan is not a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 0

        # symbolic args
        s = proj.factory.entry_state(args=["aaa", claripy.BVS("arg_2", 50 * 8)], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        found = xpl.found[0]
        conc = found.solver.eval(found.memory.load(found.registers.load("sp"), 400), cast_to=bytes)

        assert b"Yan is a noob" in conc

    def test_arm(self):
        proj = angr.Project(os.path.join(test_location, "armel", "argv_test"), auto_load_libs=False)
        r_addr = 0x1048C

        s = proj.factory.entry_state(args=["aaa", "Yan is a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 1

        s = proj.factory.entry_state(args=["aaa", "Yan is not a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 0

        # symbolic args
        s = proj.factory.entry_state(args=["aaa", claripy.BVS("arg_2", 50 * 8)], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        found = xpl.found[0]
        conc = found.solver.eval(found.memory.load(found.registers.load("sp"), 400), cast_to=bytes)

        assert b"Yan is a noob" in conc

    def test_ppc32(self):
        proj = angr.Project(os.path.join(test_location, "ppc", "argv_test"), auto_load_libs=False)
        r_addr = 0x10000498

        s = proj.factory.entry_state(args=["aaa", "Yan is a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 1

        s = proj.factory.entry_state(args=["aaa", "Yan is not a noob"], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        assert len(xpl.found) == 0

        # symbolic args
        s = proj.factory.entry_state(args=["aaa", claripy.BVS("arg_2", 50 * 8)], env={"HOME": "/home/angr"})
        xpl = proj.factory.simulation_manager(s).explore(find=r_addr)

        found = xpl.found[0]
        conc = found.solver.eval(found.memory.load(found.registers.load("sp"), 400), cast_to=bytes)

        assert b"Yan is a noob" in conc


if __name__ == "__main__":
    unittest.main()



# pylint:disable=consider-using-in
from typing import Optional, Callable, TYPE_CHECKING

from ailment import Block, Stmt, Expr, AILBlockWalker

from ...code_location import CodeLocation
from ...knowledge_plugins.key_definitions.constants import OP_BEFORE, OP_AFTER
from ...knowledge_plugins.key_definitions import atoms

if TYPE_CHECKING:
    from archinfo import Arch
    from .propagator import PropagatorAILState
    from angr.analyses.reaching_definitions import ReachingDefinitionsModel


class OutdatedDefinitionWalker(AILBlockWalker):
    """
    Walks an AIL expression to find outdated definitions.
    """

    def __init__(
        self,
        expr,
        expr_defat: CodeLocation,
        current_loc: CodeLocation,
        state: "PropagatorAILState",
        arch: "Arch",
        avoid: Optional[Expr.Expression] = None,
        extract_offset_to_sp: Callable = None,
        rda: "ReachingDefinitionsModel" = None,
    ):
        super().__init__()
        self.expr = expr
        self.expr_defat = expr_defat
        self.current_loc = current_loc
        self.state = state
        self.avoid = avoid
        self.arch = arch
        self.extract_offset_to_sp = extract_offset_to_sp
        self.expr_handlers[Expr.Register] = self._handle_Register
        self.expr_handlers[Expr.Load] = self._handle_Load
        self.expr_handlers[Expr.Tmp] = self._handle_Tmp
        self.expr_handlers[Expr.VEXCCallExpression] = self._handle_VEXCCallExpression
        self.out_dated = False
        self.has_avoid = False
        self.rda = rda

    # pylint:disable=unused-argument
    def _handle_Tmp(self, expr_idx: int, expr: Expr.Tmp, stmt_idx: int, stmt: Stmt.Assignment, block: Optional[Block]):
        if self.avoid is not None and expr.likes(self.avoid):
            self.has_avoid = True

    # pylint:disable=unused-argument
    def _handle_Register(
        self, expr_idx: int, expr: Expr.Register, stmt_idx: int, stmt: Stmt.Assignment, block: Optional[Block]
    ):
        if (
            self.avoid is not None
            and isinstance(self.avoid, Expr.Register)
            and (expr.likes(self.avoid) or self._reg_overlap(expr, self.avoid))
        ):
            self.has_avoid = True

        # is the used register still alive at this point?
        defs_defat = self.rda.get_defs(atoms.Register(expr.reg_offset, expr.size), self.expr_defat, OP_AFTER)
        defs_currentloc = self.rda.get_defs(atoms.Register(expr.reg_offset, expr.size), self.current_loc, OP_BEFORE)

        codelocs_defat = {def_.codeloc for def_ in defs_defat}
        codelocs_currentloc = {def_.codeloc for def_ in defs_currentloc}
        if not (codelocs_defat and codelocs_currentloc and codelocs_defat == codelocs_currentloc):
            self.out_dated = True

    def _handle_Load(self, expr_idx: int, expr: Expr.Load, stmt_idx: int, stmt: Stmt.Statement, block: Optional[Block]):
        if self.avoid is not None and (expr == self.avoid or expr.addr == self.avoid):
            self.has_avoid = True

        if isinstance(expr.addr, Expr.StackBaseOffset):
            sp_offset = self.extract_offset_to_sp(expr.addr)

            if sp_offset is not None:
                defs_defat = self.rda.get_defs(
                    atoms.MemoryLocation(atoms.SpOffset(expr.bits, sp_offset), expr.size), self.expr_defat, OP_AFTER
                )
                defs_currentloc = self.rda.get_defs(
                    atoms.MemoryLocation(atoms.SpOffset(expr.bits, sp_offset), expr.size), self.current_loc, OP_BEFORE
                )

                codelocs_defat = {def_.codeloc for def_ in defs_defat}
                codelocs_currentloc = {def_.codeloc for def_ in defs_currentloc}

                if not codelocs_defat and not codelocs_currentloc:
                    # fallback
                    if not self._check_store_precedes_load(self.expr_defat, self.current_loc):
                        self.out_dated = True
                elif not (codelocs_defat and codelocs_currentloc and codelocs_defat == codelocs_currentloc):
                    self.out_dated = True

            else:
                # in cases where expr.addr cannot be resolved to a concrete stack offset, we play safe and assume
                # it's outdated
                self.out_dated = True

        elif isinstance(expr.addr, Expr.Const):
            mem_addr = expr.addr.value

            defs_defat = self.rda.get_defs(atoms.MemoryLocation(mem_addr, expr.size), self.expr_defat, OP_AFTER)
            defs_currentloc = self.rda.get_defs(atoms.MemoryLocation(mem_addr, expr.size), self.current_loc, OP_BEFORE)

            codelocs_defat = {def_.codeloc for def_ in defs_defat}
            codelocs_currentloc = {def_.codeloc for def_ in defs_currentloc}

            if codelocs_defat != codelocs_currentloc:
                self.out_dated = True

        else:
            # the address is not concrete - we check the address first
            super()._handle_Load(expr_idx, expr, stmt_idx, stmt, block)
            # then if the address expression is up-to-date, we check the global store
            if not self.out_dated:
                if (
                    self.state.global_stores
                    and not all(
                        self._check_store_precedes_load(CodeLocation(store_block_addr, store_stmt_idx), self.expr_defat)
                        for store_block_addr, store_stmt_idx, addr, store in self.state.global_stores
                    )
                    or self.state.last_stack_store is not None
                    and not self._check_store_precedes_load(
                        CodeLocation(*self.state.last_stack_store[:2]), self.expr_defat
                    )
                ):
                    self.out_dated = True

    def _handle_VEXCCallExpression(
        self, expr_idx: int, expr: Expr.VEXCCallExpression, stmt_idx: int, stmt: Stmt.Statement, block: Optional[Block]
    ):
        if self.avoid is not None:
            if any(op == self.avoid for op in expr.operands):
                self.has_avoid = True

        super()._handle_VEXCCallExpression(expr_idx, expr, stmt_idx, stmt, block)

    @staticmethod
    def _reg_overlap(reg0: Expr.Register, reg1: Expr.Register) -> bool:
        if reg0.reg_offset <= reg1.reg_offset < reg0.reg_offset + reg0.size:
            return True
        if reg1.reg_offset <= reg0.reg_offset < reg1.reg_offset + reg1.size:
            return True
        return False

    @staticmethod
    def _check_store_precedes_load(store_defat: Optional[CodeLocation], load_defat: Optional[CodeLocation]) -> bool:
        """
        Check if store precedes load based on their AIL statement IDs.
        """
        if store_defat is None or load_defat is None:
            return True
        return store_defat.block_addr == load_defat.block_addr and store_defat.stmt_idx <= load_defat.stmt_idx



from ailment.statement import ConditionalJump
from ailment.expression import ITE

from .base import PeepholeOptimizationStmtBase


class CoalesceSameCascadingIfs(PeepholeOptimizationStmtBase):
    __slots__ = ()

    NAME = "Coalescing cascading If constructs"
    stmt_classes = (ConditionalJump,)

    def optimize(self, stmt: ConditionalJump, stmt_idx: int = None, block=None, **kwargs):
        cond = stmt.condition

        # if (cond) {ITE(cond, true_branch, false_branch)} else {} ==> if (cond) {true_branch} else {}
        if isinstance(stmt.true_target, ITE) and cond == stmt.true_target.cond:
            new_true_target = stmt.true_target.iftrue
        else:
            new_true_target = stmt.true_target

        if cond is not stmt.condition or new_true_target is not stmt.true_target:
            # it's updated
            return ConditionalJump(
                stmt.idx, cond, new_true_target, stmt.false_target, false_target_idx=stmt.false_target_idx, **stmt.tags
            )
        return None



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import logging
import unittest

import pyvex
import claripy

from angr import SimState, load_shellcode
from angr.engines import HeavyVEXMixin
import angr.engines.vex.claripy.ccall as s_ccall

l = logging.getLogger(__name__)


class TestVex(unittest.TestCase):
    def test_ccall(self):
        s = SimState(arch="AMD64")

        l.debug("Testing amd64_actions_ADD")
        l.debug("(8-bit) 1 + 1...")
        arg_l = s.solver.BVV(1, 8)
        arg_r = s.solver.BVV(1, 8)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ADD(s, 8, arg_l, arg_r, 0, platform="AMD64")
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("(32-bit) (-1) + (-2)...")
        arg_l = s.solver.BVV(-1, 32)
        arg_r = s.solver.BVV(-1, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ADD(s, 32, arg_l, arg_r, 0, platform="AMD64")
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 1)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 1)
        assert s.solver.is_true(of == 0)

        l.debug("Testing pc_actions_SUB")
        l.debug(
            "(8-bit) 1 - 1...",
        )
        arg_l = s.solver.BVV(1, 8)
        arg_r = s.solver.BVV(1, 8)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_SUB(s, 8, arg_l, arg_r, 0, platform="AMD64")
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(pf == 1)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 1)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("(32-bit) (-1) - (-2)...")
        arg_l = s.solver.BVV(-1, 32)
        arg_r = s.solver.BVV(-2, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_SUB(s, 32, arg_l, arg_r, 0, platform="AMD64")
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("Testing pc_actions_ROL")
        l.debug("(8-bit) ROL 1 1...")
        result = s.solver.BVV(2, 8)  # the result of rol(1, 1)
        oldflags = s.solver.BVV(0, 8)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ROL(s, 8, result, None, oldflags, platform="AMD64")
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("(32-bit) ROL (-1) (-2)... (shift out of range)")
        result = s.solver.BVV(-1, 32)  # the result of rol(-1, 0xfe)
        oldflags = s.solver.BVV(0, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ROL(s, 32, result, None, oldflags, platform="AMD64")
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("Testing pc_actions_ROR")
        l.debug("(32-bit) ROR 2 1...")
        result = s.solver.BVV(1, 32)  # the result of ror(2, 1)
        oldflags = s.solver.BVV(0, 8)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ROR(s, 32, result, None, oldflags, platform="AMD64")
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("Testing pc_actions_ROR")
        l.debug("(32-bit) ROR 1 1...")
        result = s.solver.BVV(0x80000000, 32)  # the result of ror(1, 1)
        oldflags = s.solver.BVV(0, 8)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ROR(s, 32, result, None, oldflags, platform="AMD64")
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 1)

        l.debug("Testing pc_actions_ROR")
        l.debug("(32-bit) ROR -1 1...")
        result = s.solver.BVV(-1, 32)  # the result of ror(0xffffffff, 1)
        oldflags = s.solver.BVV(0, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ROR(s, 32, result, None, oldflags, platform="AMD64")
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("(32-bit) ROR (-1) (-2)... (shift out of range)")
        result = s.solver.BVV(-1, 32)  # the result of ror(0xffffffff, 0xfe)
        oldflags = s.solver.BVV(0, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ROR(s, 32, result, None, oldflags, platform="AMD64")
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(pf == 0)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 0)

        l.debug("Testing pc_actions_SHR")
        l.debug("(64-bit) SHR 0xffffffffffffffff 1")
        result = s.solver.BVV(0x7FFFFFFFFFFFFFFF, 64)
        subshifted_result = s.solver.BVV(0xFFFFFFFFFFFFFFFF, 64)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_SHR(s, 64, result, subshifted_result, None, platform="AMD64")
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(pf == 1)
        assert s.solver.is_true(af == 0)
        assert s.solver.is_true(zf == 0)
        assert s.solver.is_true(sf == 0)
        assert s.solver.is_true(of == 1)

        l.debug("Testing amd64_actions_ADCX")

        l.debug("(ADCX, 32-bit) 0xffffffff + 1...")
        arg_l = s.solver.BVV(0xFFFFFFFF, 32)
        arg_r = s.solver.BVV(1, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ADCX(
            s, 32, arg_l, arg_r, s.solver.BVV(0, 32), True, platform="AMD64"
        )
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(of == 0)

        l.debug("(ADOX, 32-bit) 0xffffffff + 1...")
        arg_l = s.solver.BVV(0xFFFFFFFF, 32)
        arg_r = s.solver.BVV(1, 32)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ADCX(
            s, 32, arg_l, arg_r, s.solver.BVV(0, 32), False, platform="AMD64"
        )
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(of == 1)

        l.debug("(ADCX, 64-bit) 0xffffffffffffffff + 1...")
        arg_l = s.solver.BVV(0xFFFFFFFFFFFFFFFF, 64)
        arg_r = s.solver.BVV(1, 64)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ADCX(
            s, 64, arg_l, arg_r, s.solver.BVV(0, 64), True, platform="AMD64"
        )
        assert s.solver.is_true(cf == 1)
        assert s.solver.is_true(of == 0)

        l.debug("(ADOX, 64-bit) 0xffffffffffffffff + 1...")
        arg_l = s.solver.BVV(0xFFFFFFFFFFFFFFFF, 64)
        arg_r = s.solver.BVV(1, 64)
        cf, pf, af, zf, sf, of = s_ccall.pc_actions_ADCX(
            s, 64, arg_l, arg_r, s.solver.BVV(0, 64), False, platform="AMD64"
        )
        assert s.solver.is_true(cf == 0)
        assert s.solver.is_true(of == 1)

    def test_aarch64_32bit_ccalls(self):
        # GitHub issue #1238
        s = SimState(arch="AArch64")

        x = s.solver.BVS("x", 32)
        # A normal operation
        flag_z = s_ccall.arm64g_calculate_flag_z(s, s_ccall.ARM64G_CC_OP_ADD32, x, s.solver.BVV(1, 32), 0)
        assert s.satisfiable(extra_constraints=(flag_z == 0,))
        assert s.satisfiable(extra_constraints=(flag_z == 1,))
        # What VEX does
        flag_z = s_ccall.arm64g_calculate_flag_z(
            s, s_ccall.ARM64G_CC_OP_ADD32, x.zero_extend(32), s.solver.BVV(1, 64), 0
        )
        assert s.satisfiable(extra_constraints=(flag_z == 0,))
        assert s.satisfiable(extra_constraints=(flag_z == 1,))

    def test_some_vector_ops(self):
        engine = HeavyVEXMixin(None)
        s = SimState(arch="AMD64")

        def translate(state, op, args):  # pylint:disable=unused-argument
            return engine._perform_vex_expr_Op(op, args)

        a = s.solver.BVV(0xFFFF0000000100020003000400050006, 128)
        b = s.solver.BVV(0x00020002000200020002000200020002, 128)

        calc_result = translate(s, "Iop_Sub16x8", (a, b))
        correct_result = s.solver.BVV(0xFFFDFFFEFFFF00000001000200030004, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_CmpEQ16x8", (a, b))
        correct_result = s.solver.BVV(0x000000000000FFFF0000000000000000, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_CmpEQ8x16", (a, b))
        correct_result = s.solver.BVV(0x0000FF00FF00FFFFFF00FF00FF00FF00, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_CmpGT16Sx8", (a, b))
        correct_result = s.solver.BVV(0x0000000000000000FFFFFFFFFFFFFFFF, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_CmpGT16Ux8", (a, b))
        correct_result = s.solver.BVV(0xFFFF000000000000FFFFFFFFFFFFFFFF, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_InterleaveLO16x8", (a, b))
        correct_result = s.solver.BVV(0x00030002000400020005000200060002, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_InterleaveLO8x16", (a, b))
        correct_result = s.solver.BVV(0x00000302000004020000050200000602, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Min8Ux16", (a, b))
        correct_result = s.solver.BVV(0x00020000000100020002000200020002, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Min8Sx16", (a, b))
        correct_result = s.solver.BVV(0xFFFF0000000100020002000200020002, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_QNarrowBin16Sto8Ux16", (a, b))
        correct_result = s.solver.BVV(0x00000102030405060202020202020202, 128)
        assert s.solver.is_true(calc_result == correct_result)

        c = s.solver.BVV(0xFF008877, 32)
        d = s.solver.BVV(0x11111111, 32)

        calc_result = translate(s, "Iop_HAdd8Sx4", (c, d))
        correct_result = s.solver.BVV(0x0808CC44, 32)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_QAdd8Sx4", (c, d))
        correct_result = s.solver.BVV(0x1011997F, 32)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_QAdd8Ux4", (c, d))
        correct_result = s.solver.BVV(0xFF119988, 32)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_QSub8Sx4", (c, d))
        correct_result = s.solver.BVV(0xEEEF8066, 32)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_QSub8Ux4", (c, d))
        correct_result = s.solver.BVV(0xEE007766, 32)
        assert s.solver.is_true(calc_result == correct_result)

        e = s.solver.BVV(0xFF00887766554433, 64)
        f = s.solver.BVV(0x0202000200020002, 64)

        calc_result = translate(s, "Iop_QNarrowBin16Sto8Ux8", (e, f))
        correct_result = s.solver.BVV(0x0000FFFFFF020202, 64)
        assert s.solver.is_true(calc_result == correct_result)

        gg = claripy.BVV(0x111111112222222233333333FFFFFFFF, 128)
        h = claripy.BVV(0x1111111100000000FFFFFFFFFFFFFFFF, 128)

        calc_result = translate(s, "Iop_CmpEQ32Fx4", (gg, h))
        correct_result = claripy.BVV(0xFFFFFFFF000000000000000000000000, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Clz32x4", (gg,))
        correct_result = claripy.BVV(0x00000003000000020000000200000000, 128)
        assert s.solver.is_true(calc_result == correct_result)

        i = claripy.BVV(0x1001000000001000, 64)
        j = claripy.BVV(0x100000000000F000, 64)

        calc_result = translate(s, "Iop_Mull16Sx4", (i, j))
        correct_result = claripy.BVV(0x10010000000000000000000FF000000, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Mull16Ux4", (i, j))
        correct_result = claripy.BVV(0x100100000000000000000000F000000, 128)
        assert s.solver.is_true(calc_result == correct_result)

        k = claripy.BVV(0xE7, 8)
        ll = claripy.BVV(0x1234, 16)
        m = claripy.BVV(0x12345678, 32)

        calc_result = translate(s, "Iop_Dup8x8", (k,))
        correct_result = claripy.BVV(0xE7E7E7E7E7E7E7E7, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Dup8x16", (k,))
        correct_result = claripy.BVV(0xE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Dup16x4", (ll,))
        correct_result = claripy.BVV(0x1234123412341234, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Dup16x8", (ll,))
        correct_result = claripy.BVV(0x12341234123412341234123412341234, 128)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Dup32x2", (m,))
        correct_result = claripy.BVV(0x1234567812345678, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_Dup32x4", (m,))
        correct_result = claripy.BVV(0x12345678123456781234567812345678, 128)
        assert s.solver.is_true(calc_result == correct_result)

        n = claripy.BVV(0x0123456789ABCDEF, 64)
        o = claripy.BVV(0xAF, 8)
        p = claripy.BVV(0xDFEC, 16)
        q = claripy.BVV(0xBFCFDFEF, 32)
        r = claripy.BVV(0x0102030405060708, 64)
        ss = claripy.BVS("index", 8)

        # According to the source code of LibVex, the index is a U8 constant
        calc_result = translate(s, "Iop_GetElem8x8", (n, claripy.BVV(0, 8)))
        correct_result = claripy.BVV(0xEF, 8)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem8x8", (n, claripy.BVV(1, 8)))
        correct_result = claripy.BVV(0xCD, 8)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem8x8", (n, claripy.BVV(6, 8)))
        correct_result = claripy.BVV(0x23, 8)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem8x8", (n, claripy.BVV(7, 8)))
        correct_result = claripy.BVV(0x01, 8)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem16x4", (n, claripy.BVV(0, 8)))
        correct_result = claripy.BVV(0xCDEF, 16)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem16x4", (n, claripy.BVV(3, 8)))
        correct_result = claripy.BVV(0x0123, 16)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem32x2", (n, claripy.BVV(0, 8)))
        correct_result = claripy.BVV(0x89ABCDEF, 32)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem32x2", (n, claripy.BVV(1, 8)))
        correct_result = claripy.BVV(0x01234567, 32)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem8x8", (n, claripy.BVV(0, 8), o))
        correct_result = claripy.BVV(0x0123456789ABCDAF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem8x8", (n, claripy.BVV(1, 8), o))
        correct_result = claripy.BVV(0x0123456789ABAFEF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem8x8", (n, claripy.BVV(6, 8), o))
        correct_result = claripy.BVV(0x01AF456789ABCDEF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem8x8", (n, claripy.BVV(7, 8), o))
        correct_result = claripy.BVV(0xAF23456789ABCDEF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem16x4", (n, claripy.BVV(0, 8), p))
        correct_result = claripy.BVV(0x0123456789ABDFEC, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem16x4", (n, claripy.BVV(3, 8), p))
        correct_result = claripy.BVV(0xDFEC456789ABCDEF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem32x2", (n, claripy.BVV(0, 8), q))
        correct_result = claripy.BVV(0x01234567BFCFDFEF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_SetElem32x2", (n, claripy.BVV(1, 8), q))
        correct_result = claripy.BVV(0xBFCFDFEF89ABCDEF, 64)
        assert s.solver.is_true(calc_result == correct_result)

        # Symbolic indexes
        calc_result = translate(ss, "Iop_GetElem8x8", (r, ss))
        correct_result = claripy.If(
            ss == 7,
            claripy.BVV(0x01, 8),
            claripy.If(
                ss == 6,
                claripy.BVV(0x02, 8),
                claripy.If(
                    ss == 5,
                    claripy.BVV(0x03, 8),
                    claripy.If(
                        ss == 4,
                        claripy.BVV(0x04, 8),
                        claripy.If(
                            ss == 3,
                            claripy.BVV(0x05, 8),
                            claripy.If(
                                ss == 2,
                                claripy.BVV(0x06, 8),
                                claripy.If(ss == 1, claripy.BVV(0x07, 8), claripy.BVV(0x08, 8)),
                            ),
                        ),
                    ),
                ),
            ),
        )
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem16x4", (r, ss))
        correct_result = claripy.If(
            ss == 3,
            claripy.BVV(0x0102, 16),
            claripy.If(
                ss == 2,
                claripy.BVV(0x0304, 16),
                claripy.If(ss == 1, claripy.BVV(0x0506, 16), claripy.BVV(0x0708, 16)),
            ),
        )
        assert s.solver.is_true(calc_result == correct_result)

        calc_result = translate(s, "Iop_GetElem32x2", (r, ss))
        correct_result = claripy.If(ss == 1, claripy.BVV(0x01020304, 32), claripy.BVV(0x05060708, 32))
        assert s.solver.is_true(calc_result == correct_result)

        r_slices = r.chop(8)
        calc_result = translate(s, "Iop_SetElem8x8", (r, ss, o))
        correct_result = claripy.Concat(
            claripy.If(ss == 7, o, r_slices[0]),
            claripy.If(ss == 6, o, r_slices[1]),
            claripy.If(ss == 5, o, r_slices[2]),
            claripy.If(ss == 4, o, r_slices[3]),
            claripy.If(ss == 3, o, r_slices[4]),
            claripy.If(ss == 2, o, r_slices[5]),
            claripy.If(ss == 1, o, r_slices[6]),
            claripy.If(ss == 0, o, r_slices[7]),
        )
        assert s.solver.is_true(calc_result == correct_result)

        r_slices = r.chop(16)
        calc_result = translate(s, "Iop_SetElem16x4", (r, ss, p))
        correct_result = claripy.Concat(
            claripy.If(ss == 3, p, r_slices[0]),
            claripy.If(ss == 2, p, r_slices[1]),
            claripy.If(ss == 1, p, r_slices[2]),
            claripy.If(ss == 0, p, r_slices[3]),
        )
        assert s.solver.is_true(calc_result == correct_result)

        r_slices = r.chop(32)
        calc_result = translate(s, "Iop_SetElem32x2", (r, ss, q))
        correct_result = claripy.Concat(
            claripy.If(ss == 1, q, r_slices[0]),
            claripy.If(ss == 0, q, r_slices[1]),
        )
        assert s.solver.is_true(calc_result == correct_result)

    def test_store_simplification(self):
        state = SimState(arch="X86")
        state.regs.esp = state.solver.BVS("stack_pointer", 32)
        state.regs.ebp = state.solver.BVS("base_pointer", 32)
        state.regs.eax = state.solver.BVS("base_eax", 32)

        irsb = pyvex.IRSB(b"PT]\xc2\x10\x00", 0x4000, state.arch)
        sim_successors = HeavyVEXMixin(None).process(state.copy(), irsb=irsb)
        exit_state = sim_successors.all_successors[0]

        assert claripy.backends.z3.is_true(exit_state.regs.ebp == state.regs.esp - 4)

    def test_loadg_no_constraint_creation(self):
        state = SimState(arch="armel", mode="symbolic")
        engine = HeavyVEXMixin(None)

        stmt = pyvex.IRStmt.LoadG(
            "Iend_LE",
            "ILGop_16Uto32",
            0,  # dst
            pyvex.IRExpr.Const(pyvex.const.U32(0x2000)),  # addr (src)
            pyvex.IRExpr.Const(pyvex.const.U32(0x1337)),  # alt
            pyvex.IRExpr.RdTmp(1),  # guard
        )
        tyenv = pyvex.IRTypeEnv(state.arch)
        tyenv.types = ["Ity_I32", "Ity_I32"]
        state.scratch.set_tyenv(tyenv)
        state.scratch.temps[1] = state.solver.BVS("tmp_1", 32)
        engine.state = state
        engine._handle_vex_stmt(stmt)

        # LOADG should not create new constraints - it is a simple conditional memory read. The conditions should only
        # be used inside the value AST to guard the memory read.
        assert not state.solver.constraints
        assert state.scratch.temps[0] is not None
        assert state.scratch.temps[0].variables.issuperset(state.scratch.temps[1].variables)
        assert state.scratch.temps[0].op == "If"

    def test_amd64_ud012_behaviors(self):
        # Test if VEX's lifter behaves as what CFGFast expects
        #
        # Note: if such behaviors change in the future, you also need to fix the ud{0,1,2} handling logic in
        # CFGFast._generate_cfgnode().

        # according to VEX, ud0 is not part of the block
        a = load_shellcode(b"\x90\x90\x0f\xff", "amd64")
        block_0 = a.factory.block(0)
        assert block_0.size == 2

        # according to VEX, ud1 is not part of the block
        a = load_shellcode(b"\x90\x90\x0f\xb9", "amd64")
        block_1 = a.factory.block(0)
        assert block_1.size == 2

        # according to VEX, ud2 under AMD64 *is* part of the block
        a = load_shellcode(b"\x90\x90\x0f\x0b", "amd64")
        block_2 = a.factory.block(0)
        assert block_2.size == 4

    def test_x86_ud2_behaviors(self):
        # Test if VEX's lifter behaves as what CFGFast expects
        #
        # Note: if such behaviors change in the future, you also need to fix the ud2 handling logic in
        # CFGFast._generate_cfgnode().

        # according to VEX, ud2 on x86 is not part of the block
        a = load_shellcode(b"\x90\x90\x0f\x0b", "x86")
        block_0 = a.factory.block(0)
        assert block_0.size == 2

    def test_blsr(self):
        p = load_shellcode(bytes.fromhex("c4e2f8f3cf0f95c00fb6c0c3"), arch="amd64")
        # compiled form of !!(x & (x - 1))
        # 2c0:   c4 e2 f8 f3 cf          blsr   %rdi,%rax

        # 2c5:   0f 95 c0                setne  %al
        # 2c8:   0f b6 c0                movzbl %al,%eax
        # 2cb:   c3                      retq
        x = claripy.BVS("x", 64)
        s = p.factory.call_state(0, x)
        sm = p.factory.simulation_manager(s)
        sm.run()

        target_func = claripy.If((x & (x - 1)) == 0, claripy.BVV(0, 64), 1)
        solver = claripy.Solver()
        solver.add(sm.one_deadended.regs.rax != target_func)
        assert not solver.satisfiable()


if __name__ == "__main__":
    unittest.main()



# pylint:disable=no-else-break
import logging
from typing import Optional, Tuple, List

from ...errors import AngrCFGError


l = logging.getLogger(name=__name__)


class Segment:
    """
    Representing a memory block. This is not the "Segment" in ELF memory model
    """

    __slots__ = ["start", "end", "sort"]

    def __init__(self, start, end, sort):
        """
        :param int start:   Start address.
        :param int end:     End address.
        :param str sort:    Type of the segment, can be code, data, etc.
        :return: None
        """

        self.start = start
        self.end = end
        self.sort = sort

    def __repr__(self):
        s = f"[{self.start:#x}-{self.end:#x}, {self.sort}]"
        return s

    @property
    def size(self):
        """
        Calculate the size of the Segment.

        :return: Size of the Segment.
        :rtype: int
        """
        return self.end - self.start

    def copy(self):
        """
        Make a copy of the Segment.

        :return: A copy of the Segment instance.
        :rtype: angr.analyses.cfg_fast.Segment
        """
        return Segment(self.start, self.end, self.sort)


class SegmentList:
    """
    SegmentList describes a series of segmented memory blocks. You may query whether an address belongs to any of the
    blocks or not, and obtain the exact block(segment) that the address belongs to.
    """

    __slots__ = ["_list", "_bytes_occupied"]

    def __init__(self):
        self._list: List[Segment] = []
        self._bytes_occupied = 0

    #
    # Overridden methods
    #

    def __len__(self):
        return len(self._list)

    def __getitem__(self, idx: int) -> Segment:
        return self._list[idx]

    #
    # Private methods
    #

    def _insert_and_merge(self, address: int, size: int, sort: str, idx: int) -> None:
        """
        Determines whether the block specified by (address, size) should be merged with adjacent blocks.

        :param address: Starting address of the block to be merged.
        :param size: Size of the block to be merged.
        :param sort: Type of the block.
        :param idx: ID of the address.
        """

        # sanity check
        if idx > 0 and address + size <= self._list[idx - 1].start:
            # There is a bug, since _list[idx] must be the closest one that is less than the current segment
            l.warning("BUG FOUND: new segment should always be greater than _list[idx].")
            # Anyways, let's fix it.
            self._insert_and_merge(address, size, sort, idx - 1)
            return

        # Insert the block first
        # The new block might be overlapping with other blocks. _insert_and_merge_core will fix the overlapping.
        if idx == len(self._list):
            self._list.append(Segment(address, address + size, sort))
        else:
            self._list.insert(idx, Segment(address, address + size, sort))
        # Apparently _bytes_occupied will be wrong if the new block overlaps with any existing block. We will fix it
        # later
        self._bytes_occupied += size

        # Search forward to merge blocks if necessary
        pos = idx
        while pos < len(self._list):
            merged, pos, bytes_change = self._insert_and_merge_core(pos, "forward")

            if not merged:
                break

            self._bytes_occupied += bytes_change

        # Search backward to merge blocks if necessary
        pos = idx

        while pos > 0:
            merged, pos, bytes_change = self._insert_and_merge_core(pos, "backward")

            if not merged:
                break

            self._bytes_occupied += bytes_change

    def _insert_and_merge_core(self, pos: int, direction: str):
        """
        The core part of method _insert_and_merge.

        :param pos:         The starting position.
        :param direction:   If we are traversing forwards or backwards in the list. It determines where the "sort"
                                of the overlapping memory block comes from. If everything works as expected, "sort" of
                                the overlapping block is always equal to the segment occupied most recently.
        :return: A tuple of (merged (bool), new position to begin searching (int), change in total bytes (int)
        :rtype: tuple
        """

        bytes_changed = 0

        if direction == "forward":
            if pos == len(self._list) - 1:
                return False, pos, 0
            previous_segment = self._list[pos]
            previous_segment_pos = pos
            segment = self._list[pos + 1]
            segment_pos = pos + 1
        else:  # if direction == "backward":
            if pos == 0:
                return False, pos, 0
            segment = self._list[pos]
            segment_pos = pos
            previous_segment = self._list[pos - 1]
            previous_segment_pos = pos - 1

        merged = False
        new_pos = pos

        if segment.start <= previous_segment.end:
            # we should always have new_start+new_size >= segment.start

            if segment.sort == previous_segment.sort:
                # They are of the same sort - we should merge them!
                new_end = max(previous_segment.end, segment.start + segment.size)
                new_start = min(previous_segment.start, segment.start)
                new_size = new_end - new_start
                self._list[segment_pos] = Segment(new_start, new_end, segment.sort)
                self._list.pop(previous_segment_pos)
                bytes_changed = -(segment.size + previous_segment.size - new_size)

                merged = True
                new_pos = previous_segment_pos

            else:
                # Different sorts. It's a bit trickier.
                if segment.start == previous_segment.end:
                    # They are adjacent. Just don't merge.
                    pass
                else:
                    # They are overlapping. We will create one, two, or three different blocks based on how they are
                    # overlapping
                    new_segments = []
                    if segment.start < previous_segment.start:
                        new_segments.append(Segment(segment.start, previous_segment.start, segment.sort))

                        sort = previous_segment.sort if direction == "forward" else segment.sort
                        new_segments.append(Segment(previous_segment.start, previous_segment.end, sort))

                        if segment.end < previous_segment.end:
                            new_segments.append(Segment(segment.end, previous_segment.end, previous_segment.sort))
                        elif segment.end > previous_segment.end:
                            new_segments.append(Segment(previous_segment.end, segment.end, segment.sort))
                    else:  # segment.start >= previous_segment.start
                        if segment.start > previous_segment.start:
                            new_segments.append(Segment(previous_segment.start, segment.start, previous_segment.sort))
                        sort = previous_segment.sort if direction == "forward" else segment.sort
                        if segment.end > previous_segment.end:
                            new_segments.append(Segment(segment.start, previous_segment.end, sort))
                            new_segments.append(Segment(previous_segment.end, segment.end, segment.sort))
                        elif segment.end < previous_segment.end:
                            new_segments.append(Segment(segment.start, segment.end, sort))
                            new_segments.append(Segment(segment.end, previous_segment.end, previous_segment.sort))
                        else:
                            new_segments.append(Segment(segment.start, segment.end, sort))

                    # merge segments in new_segments array if they are of the same sort
                    i = 0
                    while len(new_segments) > 1 and i < len(new_segments) - 1:
                        s0 = new_segments[i]
                        s1 = new_segments[i + 1]
                        if s0.sort == s1.sort:
                            new_segments = (
                                new_segments[:i] + [Segment(s0.start, s1.end, s0.sort)] + new_segments[i + 2 :]
                            )
                        else:
                            i += 1

                    # Put new segments into self._list
                    old_size = sum([seg.size for seg in self._list[previous_segment_pos : segment_pos + 1]])
                    new_size = sum([seg.size for seg in new_segments])
                    bytes_changed = new_size - old_size

                    self._list = self._list[:previous_segment_pos] + new_segments + self._list[segment_pos + 1 :]

                    merged = True

                    if direction == "forward":
                        new_pos = previous_segment_pos + len(new_segments) - 1
                    else:
                        new_pos = previous_segment_pos

        return merged, new_pos, bytes_changed

    def _remove(self, init_address: int, init_size: int, init_idx: int) -> None:
        address = init_address
        size = init_size
        idx = init_idx

        while idx < len(self._list):
            segment = self._list[idx]
            if segment.start <= address:
                if address < segment.start + segment.size < address + size:
                    # |---segment---|
                    #      |---address + size---|
                    # shrink segment
                    segment.end = address
                    # adjust address
                    new_address = segment.start + segment.size
                    # adjust size
                    size = address + size - new_address
                    address = new_address
                    # update idx
                    idx = self.search(address)
                elif address < segment.start + segment.size and address + size <= segment.start + segment.size:
                    # |--------segment--------|
                    #    |--address + size--|
                    # break segment
                    seg0 = Segment(segment.start, address, segment.sort)
                    seg1 = Segment(address + size, segment.start + segment.size, segment.sort)
                    # remove the current segment
                    self._list.remove(segment)
                    if seg1.size > 0:
                        self._list.insert(idx, seg1)
                    if seg0.size > 0:
                        self._list.insert(idx, seg0)
                    # done
                    break
                else:
                    raise RuntimeError("Unreachable reached")
            else:  # if segment.start > address
                if address + size <= segment.start:
                    #                      |--- segment ---|
                    # |-- address + size --|
                    # no overlap
                    break
                elif segment.start < address + size <= segment.start + segment.size:
                    #            |---- segment ----|
                    # |-- address + size --|
                    #
                    # update the start of the segment
                    segment.start = address + size
                    if segment.size == 0:
                        # remove the segment
                        self._list.remove(segment)
                    break
                elif address + size > segment.start + segment.size:
                    #            |---- segment ----|
                    # |--------- address + size ----------|
                    self._list.remove(segment)
                    new_address = segment.end
                    size = address + size - new_address
                    address = new_address
                    idx = self.search(address)
                else:
                    raise RuntimeError("Unreachable reached")

    def _dbg_output(self):
        """
        Returns a string representation of the segments that form this SegmentList

        :return: String representation of contents
        :rtype: str
        """
        s = "["
        lst = []
        for segment in self._list:
            lst.append(repr(segment))
        s += ", ".join(lst)
        s += "]"
        return s

    def _debug_check(self):
        """
        Iterates over list checking segments with same sort do not overlap

        :raise: Exception: if segments overlap space with same sort
        """
        # old_start = 0
        old_end = 0
        old_sort = ""
        for segment in self._list:
            if segment.start <= old_end and segment.sort == old_sort:
                raise AngrCFGError("Error in SegmentList: blocks are not merged")
            # old_start = start
            old_end = segment.end
            old_sort = segment.sort

    #
    # Public methods
    #

    def search(self, addr: int) -> int:
        """
        Checks which segment that the address `addr` should belong to, and, returns the offset of that segment.
        Note that the address may not actually belong to the block.

        :param addr: The address to search
        :return: The offset of the segment.
        """

        start = 0
        end = len(self._list)

        while start != end:
            mid = (start + end) // 2

            segment = self._list[mid]
            if addr < segment.start:
                end = mid
            elif addr >= segment.end:
                start = mid + 1
            else:
                # Overlapped :(
                start = mid
                break

        return start

    def next_free_pos(self, address):
        """
        Returns the next free position with respect to an address, including that address itself

        :param address: The address to begin the search with (including itself)
        :return: The next free position
        """

        idx = self.search(address)
        if idx < len(self._list) and self._list[idx].start <= address < self._list[idx].end:
            # Occupied
            i = idx
            while i + 1 < len(self._list) and self._list[i].end == self._list[i + 1].start:
                i += 1
            if i == len(self._list):
                return self._list[-1].end

            return self._list[i].end

        return address

    def next_pos_with_sort_not_in(self, address, sorts, max_distance=None):
        """
        Returns the address of the next occupied block whose sort is not one of the specified ones.

        :param int address: The address to begin the search with (including itself).
        :param sorts:       A collection of sort strings.
        :param max_distance:    The maximum distance between `address` and the next position. Search will stop after
                                we come across an occupied position that is beyond `address` + max_distance. This check
                                will be disabled if `max_distance` is set to None.
        :return:            The next occupied position whose sort is not one of the specified ones, or None if no such
                            position exists.
        :rtype:             int or None
        """

        list_length = len(self._list)

        idx = self.search(address)
        if idx < list_length:
            # Occupied
            block = self._list[idx]

            if max_distance is not None and address + max_distance < block.start:
                return None

            if block.start <= address < block.end:
                # the address is inside the current block
                if block.sort not in sorts:
                    return address
                # tick the idx forward by 1
                idx += 1

            i = idx
            while i < list_length:
                if max_distance is not None and address + max_distance < self._list[i].start:
                    return None
                if self._list[i].sort not in sorts:
                    return self._list[i].start
                i += 1

        return None

    def is_occupied(self, address):
        """
        Check if an address belongs to any segment

        :param address: The address to check
        :return: True if this address belongs to a segment, False otherwise
        """

        idx = self.search(address)
        if len(self._list) <= idx:
            return False
        if self._list[idx].start <= address < self._list[idx].end:
            return True
        if idx > 0 and address < self._list[idx - 1].end:
            # TODO: It seems that this branch is never reached. Should it be removed?
            return True
        return False

    def occupied_by_sort(self, address: int) -> Optional[str]:
        """
        Check if an address belongs to any segment, and if yes, returns the sort of the segment

        :param address: The address to check
        :return: Sort of the segment that occupies this address
        """

        idx = self.search(address)
        if len(self._list) <= idx:
            return None
        if self._list[idx].start <= address < self._list[idx].end:
            return self._list[idx].sort
        if idx > 0 and address < self._list[idx - 1].end:
            # TODO: It seems that this branch is never reached. Should it be removed?
            return self._list[idx - 1].sort
        return None

    def occupied_by(self, address: int) -> Optional[Tuple[int, int, str]]:
        """
        Check if an address belongs to any segment, and if yes, returns the beginning, the size, and the sort of the
        segment.

        :param address: The address to check
        """

        idx = self.search(address)
        if len(self._list) <= idx:
            return None
        if self._list[idx].start <= address < self._list[idx].end:
            block = self._list[idx]
            return block.start, block.size, block.sort
        if idx > 0 and address < self._list[idx - 1].end:
            # TODO: It seems that this branch is never reached. Should it be removed?
            block = self._list[idx - 1]
            return block.start, block.size, block.sort
        return None

    def occupy(self, address, size, sort):
        """
        Include a block, specified by (address, size), in this segment list.

        :param int address:     The starting address of the block.
        :param int size:        Size of the block.
        :param str sort:        Type of the block.
        :return: None
        """

        if size is None or size <= 0:
            # Cannot occupy a non-existent block
            return

        # l.debug("Occpuying 0x%08x-0x%08x", address, address + size)
        if not self._list:
            self._list.append(Segment(address, address + size, sort))
            self._bytes_occupied += size
            return
        # Find adjacent element in our list
        idx = self.search(address)

        self._insert_and_merge(address, size, sort, idx)

        # self._debug_check()

    def release(self, address: int, size: int) -> None:
        """
        Remove a block, specified by (address, size), in this segment list.

        :param address: The starting address of the block.
        :param size:    Size of the block.
        """

        if size is None or size <= 0:
            # cannot release a non-existent block
            return
        if not self._list:
            return

        idx = self.search(address)
        if idx < len(self._list):
            self._remove(address, size, idx)

        # self._debug_check()

    def copy(self):
        """
        Make a copy of the SegmentList.

        :return: A copy of the SegmentList instance.
        :rtype: angr.analyses.cfg_fast.SegmentList
        """
        n = SegmentList()

        n._list = [a.copy() for a in self._list]
        n._bytes_occupied = self._bytes_occupied
        return n

    #
    # Properties
    #

    @property
    def occupied_size(self):
        """
        The sum of sizes of all blocks

        :return: An integer
        """

        return self._bytes_occupied

    @property
    def has_blocks(self):
        """
        Returns if this segment list has any block or not. !is_empty

        :return: True if it's not empty, False otherwise
        """

        return len(self._list) > 0



import angr


class Redirect(angr.SimProcedure):
    # pylint:disable=arguments-differ

    NO_RET = True

    def run(self, redirect_to=None):
        if redirect_to is None:
            raise Exception("Please specify where you wanna jump to.")

        self._custom_name = "Redirect to 0x%08x" % redirect_to
        # There is definitely no refs
        self.add_successor(self.state, redirect_to, self.state.solver.true, "Ijk_Boring")



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.sim.exec_func"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestMemFuncs(unittest.TestCase):
    def test_memmove(self):
        # auto_load_libs can't be disabled as the testcase fails
        proj = angr.Project(
            os.path.join(test_location, "x86_64", "memmove"),
            load_options={"auto_load_libs": True},
            exclude_sim_procedures_list=["memmove"],
        )
        explorer = proj.factory.simulation_manager().explore(find=[0x4005D7])
        s = explorer.found[0]
        result = s.solver.eval(s.memory.load(s.registers.load(16, 8), 13), cast_to=bytes)
        assert result == b"very useful.\x00"

    def test_memcpy(self):
        # auto_load_libs can't be disabled as the testcase fails
        proj = angr.Project(
            os.path.join(test_location, "x86_64", "memcpy"),
            load_options={"auto_load_libs": True},
            exclude_sim_procedures_list=["memcpy"],
        )
        explorer = proj.factory.simulation_manager().explore(find=[0x40065A])
        s = explorer.found[0]
        result = s.solver.eval(s.memory.load(s.registers.load(16, 8), 19), cast_to=bytes)
        assert result == b"let's test memcpy!\x00"

    def test_memset(self):
        # auto_load_libs can't be disabled as the testcase fails
        proj = angr.Project(
            os.path.join(test_location, "x86_64", "memset"),
            load_options={"auto_load_libs": True},
            exclude_sim_procedures_list=["memset"],
        )
        explorer = proj.factory.simulation_manager().explore(find=[0x400608])
        s = explorer.found[0]
        result = s.solver.eval(s.memory.load(s.registers.load(16, 8), 50), cast_to=bytes)
        assert result == b"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB\x00"


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.factory.block"  # pylint:disable=redefined-builtin

import os
import logging
import sys
import unittest

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")
l = logging.getLogger("angr.tests")

insn_texts = {
    "i386": b"add eax, 0xf",
    "x86_64": b"add rax, 0xf",
    "ppc": b"addi %r1, %r1, 0xf",
    "armel": b"add r1, r1, 0xf",
    "armel_thumb": b"add.w r1, r1, #0xf",
    "mips": b"addi $1, $1, 0xf",
}


class TestKeyStone(unittest.TestCase):
    def _run_keystone(self, arch):
        proj_arch = arch
        is_thumb = False
        if arch == "armel_thumb":
            is_thumb = True
            proj_arch = "armel"
        p = angr.Project(os.path.join(test_location, proj_arch, "fauxware"), auto_load_libs=False)
        addr = p.loader.main_object.get_symbol("authenticate").rebased_addr

        sm = p.factory.simulation_manager()
        if arch in ["i386", "x86_64"]:
            sm.one_active.regs.eax = 3
        else:
            sm.one_active.regs.r1 = 3

        if is_thumb:
            addr |= 1
        block = p.factory.block(addr, insn_text=insn_texts[arch], thumb=is_thumb).vex

        assert block.instructions == 1

        sm.step(force_addr=addr, insn_text=insn_texts[arch], thumb=is_thumb)

        if arch in ["i386", "x86_64"]:
            assert sm.one_active.solver.eval(sm.one_active.regs.eax) == 0x12
        else:
            assert sm.one_active.solver.eval(sm.one_active.regs.r1) == 0x12

    def test_i386(self):
        # Installing keystone on Windows is currently a pain. Fix the installation first (may it pip installable) before
        # re-enabling this test on Windows.
        if not sys.platform.startswith("linux"):
            return

        self._run_keystone("i386")

    def test_x86_64(self):
        # Installing keystone on Windows is currently a pain. Fix the installation first (may it pip installable) before
        # re-enabling this test on Windows.
        if not sys.platform.startswith("linux"):
            return

        self._run_keystone("x86_64")

    def test_ppc(self):
        # Installing keystone on Windows is currently a pain. Fix the installation first (may it pip installable) before
        # re-enabling this test on Windows.
        if not sys.platform.startswith("linux"):
            return

        self._run_keystone("ppc")

    def test_armel(self):
        # Installing keystone on Windows is currently a pain. Fix the installation first (may it pip installable) before
        # re-enabling this test on Windows.
        if not sys.platform.startswith("linux"):
            return

        self._run_keystone("armel")

    def test_armel_thumb(self):
        # Installing keystone on Windows is currently a pain. Fix the installation first (may it pip installable) before
        # re-enabling this test on Windows.
        if not sys.platform.startswith("linux"):
            return

        self._run_keystone("armel_thumb")

    def test_mips(self):
        # Installing keystone on Windows is currently a pain. Fix the installation first (may it pip installable) before
        # re-enabling this test on Windows.
        if not sys.platform.startswith("linux"):
            return

        self._run_keystone("mips")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.sim.exec_insn"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestAdc(unittest.TestCase):
    def test_adc_i386(self):
        proj = angr.Project(os.path.join(test_location, "i386", "test_adc"), load_options={"auto_load_libs": False})

        start = 0x804840B
        end = 0x804842E

        state = proj.factory.blank_state(
            addr=start,
            remove_options={
                angr.options.LAZY_SOLVES,
            },
            add_options={angr.options.SYMBOLIC_WRITE_ADDRESSES},
        )

        pg = proj.factory.simulation_manager(state, veritesting=False)
        pg.explore(find=end)

        found_state = pg.found[0]
        result = found_state.solver.eval(found_state.regs.eax)
        assert result == 0x1

    def test_all(self):
        self.test_adc_i386()


if __name__ == "__main__":
    unittest.main()



from ailment.expression import Convert

from .base import PeepholeOptimizationExprBase


class RemoveCascadingConversions(PeepholeOptimizationExprBase):
    __slots__ = ()

    NAME = "Remove adjacent conversions"
    expr_classes = (Convert,)

    def optimize(self, expr: Convert, **kwargs):
        if isinstance(expr.operand, Convert):
            inner = expr.operand
            if inner.from_bits == expr.to_bits:
                return inner.operand
            return Convert(expr.idx, inner.from_bits, expr.to_bits, expr.is_signed, inner.operand, **expr.tags)

        return None



import logging

from archinfo import ArchPcode

from ...calling_conventions import (
    SimCC,
    SimRegArg,
    SimStackArg,
    DEFAULT_CC,
    register_default_cc,
    SimCCUnknown,
    default_cc,
)


l = logging.getLogger(__name__)


class SimCCM68k(SimCC):
    """
    Default CC for M68k
    """

    ARG_REGS = []  # All arguments are passed in stack
    FP_ARG_REGS = []
    STACKARG_SP_DIFF = 4  # Return address is pushed on to stack by call
    RETURN_VAL = SimRegArg("d0", 4)
    RETURN_ADDR = SimStackArg(0, 4)


class SimCCRISCV(SimCC):
    """
    Default CC for RISCV
    """

    ARG_REGS = ["a0", "a1", "a2", "a3", "a4", "a5", "a6", "a7"]
    RETURN_ADDR = SimRegArg("ra", 8)
    RETURN_VAL = SimRegArg("a0", 8)


class SimCCSPARC(SimCC):
    """
    Default CC for SPARC
    """

    ARG_REGS = ["o0", "o1"]
    RETURN_VAL = SimRegArg("o0", 8)
    RETURN_ADDR = SimRegArg("o7", 8)


class SimCCSH4(SimCC):
    """
    Default CC for SH4
    """

    ARG_REGS = ["r4", "r5"]
    RETURN_VAL = SimRegArg("r0", 4)
    RETURN_ADDR = SimRegArg("pr", 4)


class SimCCPARISC(SimCC):
    """
    Default CC for PARISC
    """

    ARG_REGS = ["r26", "r25"]
    RETURN_VAL = SimRegArg("r28", 4)
    RETURN_ADDR = SimRegArg("rp", 4)


class SimCCPowerPC(SimCC):
    """
    Default CC for PowerPC
    """

    ARG_REGS = ["r3", "r4", "r5", "r6", "r7", "r8", "r9", "r10"]
    FP_ARG_REGS = []  # TODO: ???
    STACKARG_SP_BUFF = 8
    RETURN_ADDR = SimRegArg("lr", 4)
    RETURN_VAL = SimRegArg("r3", 4)


class SimCCXtensa(SimCC):
    """
    Default CC for Xtensa
    """

    ARG_REGS = ["i2", "i3", "i4", "i5", "i6", "i7"]
    FP_ARG_REGS = []  # TODO: ???
    RETURN_ADDR = SimRegArg("a0", 4)
    RETURN_VAL = SimRegArg("o2", 4)


def register_pcode_arch_default_cc(arch: ArchPcode):
    if arch.name not in DEFAULT_CC:
        # we have a bunch of manually specified mappings
        manual_cc_mapping = {
            "68000:BE:32:default": SimCCM68k,
            "RISCV:LE:64:RV64G": SimCCRISCV,
            "RISCV:LE:64:RV64GC": SimCCRISCV,
            "sparc:BE:64:default": SimCCSPARC,
            "SuperH4:LE:32:default": SimCCSH4,
            "pa-risc:BE:32:default": SimCCPARISC,
            "PowerPC:BE:32:e200": SimCCPowerPC,
            "PowerPC:BE:32:MPC8270": SimCCPowerPC,
            "Xtensa:LE:32:default": SimCCXtensa,
        }
        if arch.name in manual_cc_mapping:
            # first attempt: manually specified mappings
            cc = manual_cc_mapping[arch.name]
        else:
            # second attempt: see if there is a calling convention for a similar architecture defined in angr
            cc = default_cc(arch.name)
            if cc is None:
                # third attempt: use SimCCUnknown
                cc = SimCCUnknown

        if cc is SimCCUnknown:
            l.warning("Unknown default cc for arch %s", arch.name)
        register_default_cc(arch.name, cc)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.state_plugins.inspect"  # pylint:disable=redefined-builtin

import os
import unittest

import pyvex
import claripy
import archinfo

import angr
from angr import SimState, BP_AFTER, BP_BEFORE, SIM_PROCEDURES, concretization_strategies
from angr.engines import ProcedureEngine, HeavyVEXMixin, SimInspectMixin

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class InspectEngine(SimInspectMixin, HeavyVEXMixin):
    pass


class TestInspect(unittest.TestCase):
    def test_inspect(self):
        class counts:  # pylint:disable=no-init
            mem_read = 0
            mem_write = 0
            reg_read = 0
            reg_write = 0
            tmp_read = 0
            tmp_write = 0
            expr = 0
            statement = 0
            instruction = 0
            constraints = 0
            variables = 0

        def act_mem_read(state):  # pylint:disable=unused-argument
            counts.mem_read += 1

        def act_mem_write(state):  # pylint:disable=unused-argument
            counts.mem_write += 1

        def act_reg_read(state):  # pylint:disable=unused-argument
            counts.reg_read += 1

        def act_reg_write(state):  # pylint:disable=unused-argument
            counts.reg_write += 1

        def act_tmp_read(state):  # pylint:disable=unused-argument
            counts.tmp_read += 1

        def act_tmp_write(state):  # pylint:disable=unused-argument
            counts.tmp_write += 1

        def act_expr(state):  # pylint:disable=unused-argument
            counts.expr += 1

        def act_statement(state):  # pylint:disable=unused-argument
            counts.statement += 1

        def act_instruction(state):  # pylint:disable=unused-argument
            counts.instruction += 1

        def act_variables(state):  # pylint:disable=unused-argument
            # print "CREATING:", state.inspect.symbolic_name
            counts.variables += 1

        #   def act_constraints(state): #pylint:disable=unused-argument
        #       counts.constraints += 1

        s = SimState(arch="AMD64", mode="symbolic")

        s.inspect.b("mem_write", when=BP_AFTER, action=act_mem_write)
        assert counts.mem_write == 0
        s.memory.store(100, s.solver.BVV(10, 32))
        assert counts.mem_write == 1

        s.inspect.b("mem_read", when=BP_AFTER, action=act_mem_read)
        s.inspect.b("mem_read", when=BP_AFTER, action=act_mem_read, mem_read_address=100)
        s.inspect.b("mem_read", when=BP_AFTER, action=act_mem_read, mem_read_address=123)
        s.inspect.b("mem_read", when=BP_BEFORE, action=act_mem_read, mem_read_length=3)
        assert counts.mem_read == 0
        s.memory.load(123, 4)
        s.memory.load(223, 3)
        assert counts.mem_read == 4

        s.inspect.b("reg_read", when=BP_AFTER, action=act_reg_read)
        assert counts.reg_read == 0
        s.registers.load(16, size=8)
        assert counts.reg_read == 1

        s.inspect.b("reg_write", when=BP_AFTER, action=act_reg_write)
        assert counts.reg_write == 0
        s.registers.store(16, s.solver.BVV(10, 32))
        assert counts.reg_write == 1
        assert counts.mem_write == 1
        assert counts.mem_read == 4
        assert counts.reg_read == 1

        s.inspect.b("tmp_read", when=BP_AFTER, action=act_tmp_read, tmp_read_num=0)
        s.inspect.b("tmp_write", when=BP_AFTER, action=act_tmp_write, tmp_write_num=0)
        s.inspect.b("expr", when=BP_AFTER, action=act_expr, expr_result=1016)
        s.inspect.b("statement", when=BP_AFTER, action=act_statement)
        s.inspect.b("instruction", when=BP_AFTER, action=act_instruction, instruction=1001)
        s.inspect.b("instruction", when=BP_AFTER, action=act_instruction, instruction=1000)
        irsb = pyvex.IRSB(b"\x90\x90\x90\x90\xeb\x0a", mem_addr=1000, arch=archinfo.ArchAMD64(), opt_level=0)
        irsb.pp()
        InspectEngine(None).process(s, irsb=irsb)
        assert counts.reg_write == 7
        assert counts.reg_read == 2
        assert counts.tmp_write == 1
        assert counts.tmp_read == 1
        assert counts.expr == 3
        assert counts.statement == 11
        assert counts.instruction == 2
        assert counts.constraints == 0
        assert counts.mem_write == 1
        assert counts.mem_read == 4

        s = SimState(arch="AMD64", mode="symbolic")
        s.inspect.b("symbolic_variable", when=BP_AFTER, action=act_variables)
        s.memory.load(0, 10)
        assert counts.variables == 1

    def test_inspect_exit(self):
        class counts:  # pylint:disable=no-init
            exit_before = 0
            exit_after = 0

        def handle_exit_before(state):
            counts.exit_before += 1
            exit_target = state.inspect.exit_target
            assert state.solver.eval(exit_target) == 0x3F8
            # change exit target
            state.inspect.exit_target = 0x41414141
            assert state.inspect.exit_jumpkind == "Ijk_Boring"
            assert state.inspect.exit_guard.is_true()

        def handle_exit_after(state):  # pylint:disable=unused-argument
            counts.exit_after += 1

        s = SimState(arch="AMD64", mode="symbolic")
        irsb = pyvex.IRSB(b"\x90\x90\x90\x90\xeb\x0a", mem_addr=1000, arch=archinfo.ArchAMD64())

        # break on exit
        s.inspect.b("exit", BP_BEFORE, action=handle_exit_before)
        s.inspect.b("exit", BP_AFTER, action=handle_exit_after)

        # step it
        succ = HeavyVEXMixin(None).process(s, irsb=irsb).flat_successors

        # check
        assert succ[0].solver.eval(succ[0].ip) == 0x41414141
        assert counts.exit_before == 1
        assert counts.exit_after == 1

    def test_inspect_syscall(self):
        class counts:  # pylint:disable=no-init
            exit_before = 0
            exit_after = 0

        def handle_syscall_before(state):
            counts.exit_before += 1
            syscall_name = state.inspect.syscall_name
            assert syscall_name == "close"

        def handle_syscall_after(state):
            counts.exit_after += 1
            syscall_name = state.inspect.syscall_name
            assert syscall_name == "close"

        s = SimState(arch="AMD64", mode="symbolic")
        # set up to call so syscall close
        s.regs.rax = 3
        s.regs.rdi = 2

        # break on syscall
        s.inspect.b("syscall", BP_BEFORE, action=handle_syscall_before)
        s.inspect.b("syscall", BP_AFTER, action=handle_syscall_after)

        # step it
        proc = SIM_PROCEDURES["posix"]["close"](is_syscall=True)
        ProcedureEngine(None).process(s, procedure=proc, ret_to=s.ip)

        # check counts
        assert counts.exit_before == 1
        assert counts.exit_after == 1

    def test_inspect_concretization(self):
        # some values for the test
        x = claripy.BVS("x", 64)
        y = claripy.BVS("y", 64)

        #
        # This tests concretization-time address redirection.
        #

        def change_symbolic_target(state):
            if state.inspect.address_concretization_action == "store":
                state.inspect.address_concretization_expr = claripy.BVV(0x1000, state.arch.bits)

        s = SimState(arch="AMD64")
        s.inspect.b("address_concretization", BP_BEFORE, action=change_symbolic_target)
        s.memory.store(x, "A")
        assert list(s.solver.eval_upto(x, 10)) == [0x1000]
        assert list(s.solver.eval_upto(s.memory.load(0x1000, 1), 10)) == [0x41]

        #
        # This tests disabling constraint adding through siminspect -- the write still happens
        #

        def dont_add_constraints(state):
            state.inspect.address_concretization_add_constraints = False

        s = SimState(arch="AMD64")
        s.inspect.b("address_concretization", BP_BEFORE, action=dont_add_constraints)
        s.memory.store(x, "A")
        assert len(s.solver.eval_upto(x, 10)) == 10

        #
        # This tests raising an exception if symbolic concretization fails (i.e., if the address
        # is too unconstrained). The write aborts.
        #

        class UnconstrainedAbort(Exception):
            def __init__(self, message, state):
                Exception.__init__(self, message)
                self.state = state

        def abort_unconstrained(state):
            print(state.inspect.address_concretization_strategy, state.inspect.address_concretization_result)
            if (
                isinstance(
                    state.inspect.address_concretization_strategy,
                    concretization_strategies.SimConcretizationStrategyRange,
                )
                and state.inspect.address_concretization_result is None
            ):
                raise UnconstrainedAbort("uh oh", state)

        s = SimState(arch="AMD64")
        s.memory.write_strategies.insert(0, concretization_strategies.SimConcretizationStrategyRange(128))
        s.memory._write_address_range = 1
        s.memory._write_address_range_approx = 1
        s.add_constraints(y == 10)
        s.inspect.b("address_concretization", BP_AFTER, action=abort_unconstrained)
        s.memory.store(y, "A")
        assert list(s.solver.eval_upto(s.memory.load(y, 1), 10)) == [0x41]

        try:
            s.memory.store(x, "A")
            print("THIS SHOULD NOT BE REACHED")
            assert False
        except UnconstrainedAbort as e:
            assert e.state.memory is s.memory

    def test_inspect_engine_process(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)

        def check_first_symbolic_fork(state):
            succs = state.inspect.sim_successors.successors
            succ_addr = [hex(s.addr) for s in succs]
            assert len(succ_addr) == 2
            assert "0x400692L" in succ_addr
            assert "0x400699L" in succ_addr
            print("Fork after:", hex(state.addr))
            print("Successors:", succ_addr)

        def check_second_symbolic_fork(state):
            succs = state.inspect.sim_successors.successors
            succ_addr = [hex(s.addr) for s in succs]
            assert len(succ_addr) == 2
            assert "0x4006dfL" in succ_addr
            assert "0x4006e6L" in succ_addr
            print("Fork after:", hex(state.addr))
            print("Successors:", succ_addr)

        def first_symbolic_fork(state):
            return hex(state.addr) == "0x40068eL" and isinstance(state.inspect.sim_engine, HeavyVEXMixin)
            # TODO: I think this latter check is meaningless with the eleventh hour refactor

        def second_symbolic_fork(state):
            return hex(state.addr) == "0x4006dbL" and isinstance(state.inspect.sim_engine, HeavyVEXMixin)

        def check_state(state):
            assert hex(state.inspect.sim_successors.addr) in ("0x40068eL", "0x4006dbL")

        state = p.factory.entry_state(addr=p.loader.find_symbol("main").rebased_addr)
        pg = p.factory.simulation_manager(state)
        state.inspect.b("engine_process", when=BP_BEFORE, action=check_state, condition=first_symbolic_fork)
        state.inspect.b(
            "engine_process", when=BP_AFTER, action=check_first_symbolic_fork, condition=first_symbolic_fork
        )
        pg.run()

        state = p.factory.entry_state(addr=p.loader.find_symbol("main").rebased_addr)
        pg = p.factory.simulation_manager(state)
        state.inspect.b("engine_process", when=BP_BEFORE, action=check_state, condition=second_symbolic_fork)
        state.inspect.b(
            "engine_process", when=BP_AFTER, action=check_second_symbolic_fork, condition=second_symbolic_fork
        )
        pg.run()


if __name__ == "__main__":
    unittest.main()



from typing import DefaultDict, Optional, List, Set, Tuple, TYPE_CHECKING

from collections import defaultdict

from angr.knowledge_plugins.key_definitions.atoms import Tmp

from .constants import ObservationPointType, OP_BEFORE, OP_AFTER

if TYPE_CHECKING:
    from .definition import Definition
    from angr.code_location import CodeLocation


LocationType = Tuple[int, Optional[int], Optional[int]]  # block addr, block ID, stmt ID
LocationWithPosType = Tuple[
    int, Optional[int], Optional[int], ObservationPointType
]  # block addr, block ID, stmt ID, before/after


class Liveness:
    """
    This class stores liveness information for each definition.
    """

    def __init__(self):
        self.curr_live_defs: Set["Definition"] = set()
        self.curr_loc: Optional[LocationType] = None
        self.def_to_liveness: DefaultDict["Definition", Set[LocationType]] = defaultdict(set)
        self.loc_to_defs: DefaultDict[LocationWithPosType, Set["Definition"]] = defaultdict(set)
        self._node_max_stmt_id: DefaultDict[Tuple[int, Optional[int]], int] = defaultdict(int)

    def add_def(self, d: "Definition", code_loc: "CodeLocation") -> None:
        loc = (code_loc.block_addr, code_loc.block_idx, code_loc.stmt_idx)
        self.curr_live_defs.add(d)
        self.def_to_liveness[d].add(loc)

    def kill_def(self, d: "Definition") -> None:
        self.curr_live_defs.discard(d)

    def complete_loc(self) -> None:
        if self.curr_loc is not None:
            for live_def in self.curr_live_defs:
                self.def_to_liveness[live_def].add(self.curr_loc)
            self.loc_to_defs[self.curr_loc + (OP_AFTER,)] |= self.curr_live_defs

    def at_new_stmt(self, code_loc: "CodeLocation") -> None:
        """
        Only support moving from a statement to the next statement within one basic block.
        """
        self.complete_loc()
        self.curr_loc = code_loc.block_addr, code_loc.block_idx, code_loc.stmt_idx
        if (
            code_loc.stmt_idx is not None
            and code_loc.stmt_idx > self._node_max_stmt_id[(code_loc.block_addr, code_loc.block_idx)]
        ):
            self._node_max_stmt_id[(code_loc.block_addr, code_loc.block_idx)] = code_loc.stmt_idx

    def at_new_block(self, code_loc: "CodeLocation", pred_codelocs: List["CodeLocation"]) -> None:
        """
        Only support moving to a new block from one or more blocks.
        """
        loc = code_loc.block_addr, code_loc.block_idx, code_loc.stmt_idx
        key = code_loc.block_addr, code_loc.block_idx, code_loc.stmt_idx, OP_BEFORE
        for pred_codeloc in pred_codelocs:
            if pred_codeloc.stmt_idx is None:
                # external code location
                pred_max_stmt_id = None
            else:
                pred_max_stmt_id = self._node_max_stmt_id[(pred_codeloc.block_addr, pred_codeloc.block_idx)]
            pred_key = pred_codeloc.block_addr, pred_codeloc.block_idx, pred_max_stmt_id, OP_AFTER
            all_pred_defs = self.loc_to_defs[pred_key]

            # remove tmp defs
            pred_defs = set()
            for pred_def in all_pred_defs:
                if not isinstance(pred_def.atom, Tmp):
                    pred_defs.add(pred_def)
            for pred_def in pred_defs:
                self.def_to_liveness[pred_def].add(loc)
            self.loc_to_defs[key] |= pred_defs

        self.curr_live_defs = set(self.loc_to_defs[key])
        self.curr_loc = loc

    def find_defs_at(self, code_loc: "CodeLocation", op: int = OP_BEFORE) -> Set["Definition"]:
        if op == OP_BEFORE:
            if code_loc.stmt_idx != 0:
                loc = code_loc.block_addr, code_loc.block_idx, code_loc.stmt_idx - 1, OP_AFTER
            else:
                loc = code_loc.block_addr, code_loc.block_idx, 0, OP_BEFORE
        else:
            loc = code_loc.block_addr, code_loc.block_idx, code_loc.stmt_idx, OP_AFTER
        return set() if loc not in self.loc_to_defs else self.loc_to_defs[loc]

    def copy(self) -> "Liveness":
        o = Liveness()
        o.curr_live_defs = self.curr_live_defs.copy()
        o.curr_loc = self.curr_loc
        o.def_to_liveness = self.def_to_liveness.copy()
        o.loc_to_defs = self.loc_to_defs.copy()
        o._node_max_stmt_id = self._node_max_stmt_id.copy()
        return o



import logging

from angr.procedures.stubs.format_parser import FormatParser

l = logging.getLogger(name=__name__)


class sprintf(FormatParser):
    # pylint:disable=arguments-differ

    def run(self, dst_ptr, fmt):  # pylint:disable=unused-argument
        # The format str is at index 1
        fmt_str = self._parse(fmt)
        out_str = fmt_str.replace(self.va_arg)
        self.state.memory.store(dst_ptr, out_str)

        # place the terminating null byte
        self.state.memory.store(
            dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, self.arch.byte_width)
        )

        return out_str.size() // self.arch.byte_width



import logging

import angr


l = logging.getLogger(name=__name__)

# BOOL GetProcessAffinityMask(
#   HANDLE     hProcess,
#   PDWORD_PTR lpProcessAffinityMask,
#   PDWORD_PTR lpSystemAffinityMask
# );


class GetProcessAffinityMask(angr.SimProcedure):
    paffinity_mask = None
    saffinity_mask = None

    def run(self, _, lpProcessAffinityMask, lpSystemAffinityMask):  # pylint:disable=arguments-differ
        self.fill_symbolic()
        l.info("Setting symbolic memory at %s %s", str(lpProcessAffinityMask), str(lpSystemAffinityMask))

        self.state.mem[lpProcessAffinityMask].dword = self.paffinity_mask
        self.state.mem[lpSystemAffinityMask].dword = self.saffinity_mask

        return 1

    def fill_symbolic(self):
        self.paffinity_mask = self.state.solver.BVS("lpProcessAffinityMask", 32, key=("api", "lpProcessAffinityMask"))
        self.saffinity_mask = self.state.solver.BVS("lpSystemAffinityMask", 32, key=("api", "lpSystemAffinityMask"))

    def fill_concrete(self):
        self.paffinity_mask = self.state.solver.BVV(3, 32)
        self.saffinity_mask = self.state.solver.BVV(3, 32)



from typing import Union

from .undefined import Undefined


class HeapAddress:
    """
    The representation of an address on the heap.
    """

    def __init__(self, value: Union[int, Undefined]):
        self._value = value

    @property
    def value(self):
        return self._value

    def __repr__(self):
        address_as_string = ("%#x" % self._value) if isinstance(self._value, int) else ("%s" % self._value)
        return "HeapAddress<%s>" % address_as_string

    def __add__(self, value):
        if not isinstance(value, int):
            raise TypeError("Can only add int to HeapAddress, got %s" % type(value).__name__)
        return HeapAddress(self.value + value)

    def __radd__(self, value):
        return self.__add__(value)

    def __eq__(self, other):
        return isinstance(other, HeapAddress) and self._value == other._value

    def __hash__(self):
        return hash(self._value)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.sim"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestEcho(unittest.TestCase):
    def _run_echo_haha(self, arch):
        # auto_load_libs can't be disabled as the test fails
        p = angr.Project(os.path.join(test_location, arch, "echo"), use_sim_procedures=False)
        s = p.factory.full_init_state(
            mode="symbolic_approximating", args=["echo", "haha"], add_options={angr.options.STRICT_PAGE_ACCESS}
        )
        pg = p.factory.simulation_manager(s)
        pg.run(until=lambda lpg: len(lpg.active) != 1)

        assert len(pg.deadended) == 1
        assert len(pg.active) == 0
        # Need to dump by path because the program closes stdout
        assert pg.deadended[0].posix.stdout.concretize() == [b"haha\n"]

    def test_echo_haha(self):
        self._run_echo_haha("x86_64")


if __name__ == "__main__":
    unittest.main()



import random

from ..func import Func, TestData


def rand_str(length, byte_list=None):
    if byte_list is None:
        return "".join(chr(random.randint(0, 255)) for _ in range(length))
    return "".join(random.choice(byte_list) for _ in range(length))


class strncmp(Func):
    non_null = [chr(i) for i in range(1, 256)]

    def __init__(self):
        super().__init__()  # pylint disable=useless-super-delegation

    def get_name(self):
        return "strncmp"

    def num_args(self):
        return 3

    def args(self):  # pylint disable=no-self-use
        return ["buf1", "buf2", "len"]

    def gen_input_output_pair(self):
        l = 5
        rand_str(l, strncmp.non_null)  # s
        return None

    def can_call_other_funcs(self):
        return False

    def pre_test(self, func, runner):
        # todo we don't test which order it returns the signs in
        bufa = "asdf\x00"
        bufb = "asdf\x00"
        test_input = [bufa, bufb, 5]
        test_output = [bufa, bufb, 5]
        max_steps = 10
        return_val = None
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None or s.solver.eval(s.regs.eax) != 0:
            return False

        # should return true for strcmp, false for memcpy
        bufa = "asdfa\x00sfdadfsa"
        bufb = "asdfa\x00sadfsadf"
        test_input = [bufa, bufb, 10]
        test_output = [bufa, bufb, 10]
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None:
            return False
        outval1 = s.solver.eval(s.regs.eax)

        # should fail
        bufa = "asdfc\x00as"
        bufb = "asdfb\x0011232"
        test_input = [bufa, bufb, 10]
        test_output = [bufa, bufb, 10]
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None:
            return False
        outval2 = s.solver.eval(s.regs.eax)

        # should prevent us from misidentifying strcasecmp
        bufa = "ASDFC\x00"
        bufb = "asdfc\x00"
        test_input = [bufa, bufb, 5]
        test_output = [bufa, bufb, 5]
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None:
            return False
        outval3 = s.solver.eval(s.regs.eax)

        # should distinguish strncmp and strcmp
        bufa = "abc5555"
        bufb = "abc6666"
        test_input = [bufa, bufb, 3]
        test_output = [bufa, bufb, 3]
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None:
            return False
        outval4 = s.solver.eval(s.regs.eax)

        # should distinguish strncmp and strcmp
        bufa = "abc5555"
        bufb = "abc6666"
        test_input = [bufa, bufb, 6]
        test_output = [bufa, bufb, 6]
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None:
            return False
        outval5 = s.solver.eval(s.regs.eax)

        return outval1 == 0 and outval2 != 0 and outval3 != 0 and outval4 == 0 and outval5 != 0
